/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 21, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: ObjBenchLogger.java,v 1.1 2006/01/23 16:05:52 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob;

import org.eclipse.core.runtime.*;

/**
 * A convenience logger for the Object Bench.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class ObjBenchLogger {

    /**
     * Create a status object representing the specified information.
     * 
     * @param severity
     *            the severity; one of the following: <code>IStatus.OK</code>,
     *            <code>IStatus.ERROR</code>,<code>IStatus.INFO</code>,
     *            or <code>IStatus.WARNING</code>.
     * @param code
     *            the plug-in-specific status code, or <code>OK</code>.
     * @param message
     *            a human-readable message, localized to the current locale.
     * @param exception
     *            a low-level exception, or <code>null</code> if not
     *            applicable.
     * @return the status object (not <code>null</code>).
     */
    public static IStatus createStatus(int severity, int code, String message,
            Throwable exception) {
        return new Status(severity, ObjBenchPlugin.getDefault().getBundle()
                .getSymbolicName(), code, message, exception);
    }

    /**
     * Log the specified information.
     * 
     * @param severity
     *            the severity; one of the following: <code>IStatus.OK</code>,
     *            <code>IStatus.ERROR</code>,<code>IStatus.INFO</code>,
     *            or <code>IStatus.WARNING</code>.
     * @param code
     *            the plug-in-specific status code, or <code>OK</code>.
     * @param message
     *            a human-readable message, localized to the current locale.
     * @param exception
     *            a low-level exception, or <code>null</code> if not
     *            applicable.
     */
    public static void log(int severity, int code, String message,
            Throwable exception) {
        log(createStatus(severity, code, message, exception));
    }

    /**
     * Log the given status.
     * 
     * @param status
     *            the status to log.
     */
    public static void log(IStatus status) {
        ObjBenchPlugin.getDefault().getLog().log(status);
    }

    /**
     * Log the specified error.
     * 
     * @param message
     *            a human-readable message, localized to the current locale.
     * @param exception
     *            a low-level exception, or <code>null</code> if not
     *            applicable.
     */
    public static void logError(String message, Throwable exception) {
        log(IStatus.ERROR, IStatus.OK, message, exception);
    }

    /**
     * Log the specified error.
     * 
     * @param exception
     *            a low-level exception.
     */
    public static void logError(Throwable exception) {
        String message;
        if (null == exception
                || null == (message = exception.getLocalizedMessage()))
            message = ObjBenchPlugin
                    .getResourceString("ObjBenchLogger.Unexpected.Exception"); //$NON-NLS-1$
        logError(message, exception);
    }

    /**
     * Log the specified information.
     * 
     * @param message
     *            a human-readable message, localized to the current locale.
     */
    public static void logInfo(String message) {
        log(IStatus.INFO, IStatus.OK, message, null);
    }

    /**
     * Log the specified warning. Passes along a null exception.
     * 
     * @param message
     *            a human-readable message, localized to the current locale.
     */
    public static void logWarning(String message) {
        log(IStatus.INFO, IStatus.OK, message, null);
    }

    /**
     * Log the specified warning.
     * 
     * @param message
     *            a human-readable message, localized to the current locale.
     * @param exception
     *            a low-level exception, or <code>null</code> if not
     *            applicable.
     */
    public static void logWarning(String message, Throwable exception) {
        log(IStatus.WARNING, IStatus.OK, message, exception);
    }

    /**
     * Log the specified warning.
     * 
     * @param exception
     *            a low-level exception.
     */
    public static void logWarning(Throwable exception) {
        String message;
        if (null == exception
                || null == (message = exception.getLocalizedMessage()))
            message = ObjBenchPlugin
                    .getResourceString("ObjBenchLogger.Unexpected.Exception"); //$NON-NLS-1$
        logError(message, exception);
    }
}