/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 23, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: EditPublicFieldAction.java,v 1.1 2006/01/23 16:07:15 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.actions;

import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TreeEditor;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.actions.SelectionProviderAction;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.internal.inspect.ModifiableNode;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.parser.Parser;
import edu.wpi.ebob.model.parser.ParserException;
import edu.wpi.ebob.model.parser.Scanner;

/**
 * Handles the editing of public fields.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 * @see edu.wpi.ebob.views.PropertiesView
 *  
 */
public class EditPublicFieldAction extends SelectionProviderAction {
    /** tree viewer for the inspect view */
    private TreeViewer viewer;

    /** editor for the tree viewer's tree */
    private TreeEditor editor;

    /** the currently selected node; <code>null</code> if not modifiable. */
    private ModifiableNode node;

    /**
     * Associates this action with the given viewer.
     * 
     * @param treeViewer
     *            the tree viewer in the Inspect view.
     */
    public EditPublicFieldAction(TreeViewer treeViewer) {
        super(treeViewer, ObjBenchPlugin
                .getResourceString("EditPublicFieldAction.text")); //$NON-NLS-1$
        setImageDescriptor(DebugUITools
                .getImageDescriptor(IDebugUIConstants.IMG_LCL_CHANGE_VARIABLE_VALUE));
        viewer = treeViewer;
        editor = new TreeEditor(treeViewer.getTree());
        editor.horizontalAlignment = SWT.LEFT;
        editor.grabHorizontal = true;
        editor.minimumWidth = 50;

        // start out disabled
        setEnabled(false);
    }

    /**
     * if the current selection in the tree is a public non-final field, an
     * editable text field is placed over it.
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        if (null == node)
            return;

        // get SWT-level components
        final Tree tree = viewer.getTree();
        final TreeItem item = tree.getSelection()[0];

        // clean up any previous editor control
        Control oldControl = editor.getEditor();
        if (null != oldControl)
            oldControl.dispose();

        // set up the new editor control
        final Text newControl = new Text(tree, SWT.NONE);
        newControl.setText(node.getValueAsString());
        newControl.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                newControl.dispose();
            }
        });
        newControl.addTraverseListener(new TraverseListener() {
            public void keyTraversed(TraverseEvent e) {
                switch (e.detail) {
                case SWT.TRAVERSE_RETURN:
                    Scanner scanner = new Scanner();
                    Parser parser = new Parser();
                    try {
                        node.setValue(parser.parse(scanner.scan(newControl
                                .getText())));
                        viewer.refresh(node, true);
                    } catch (ParserException pe) {
                        Throwable cause = pe.getCause();
                        if (cause instanceof RemoteException)
                            DetailsDialogUtil.openError(Display.getCurrent()
                                    .getActiveShell(), cause);
                        else
                            MessageDialog
                                    .openError(
                                            Display.getCurrent()
                                                    .getActiveShell(),
                                            ObjBenchPlugin
                                                    .getResourceString("Dialog.title.error"), pe //$NON-NLS-1$
                                                    .getLocalizedMessage());
                    }
                case SWT.TRAVERSE_ESCAPE:
                    newControl.dispose();
                }
            }
        });
        newControl.selectAll();
        newControl.setFocus();
        editor.setEditor(newControl, item);
    }

    /** gets the node currently selected in the viewer. */
    private ModifiableNode getNode(IStructuredSelection selection) {
        // determine if the selection is a public field
        if (selection.isEmpty())
            return null;
        Object obj = selection.getFirstElement();
        if (!(obj instanceof ModifiableNode))
            return null;
        final ModifiableNode tmpNode = (ModifiableNode) obj;
        if (!tmpNode.isModifiable())
            return null;
        return tmpNode;
    }

    /**
     * @see SelectionProviderAction#selectionChanged(IStructuredSelection)
     */
    public void selectionChanged(IStructuredSelection selection) {
        node = getNode(selection);
        setEnabled(null != node);
    }
}