/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 31, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: ExecuteMethodAction.java,v 1.2 2006/02/03 17:20:37 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.actions;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaElementImageDescriptor;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.internal.gui.dialogs.ExecuteMethodDialog;
import edu.wpi.ebob.internal.gui.dialogs.MethodResultDialog;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IMethod;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.parser.ParserException;
import edu.wpi.ebob.views.PropertiesView;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class ExecuteMethodAction extends Action {
    private IMethod method;

    private IObject obj;

    /**
     * size of the method icons, based Eclipse User Interface Guidelines,
     * section "Visual Design > Icon Overlays > Java Overlay".
     * 
     * @linkplain http://www.eclipse.org/articles/Article-UI-Guidelines/Contents.html#IconSpecsOvr_VDG
     */
    private static final Point METHOD_ICON_SIZE = new Point(27, 16);

    public ExecuteMethodAction(IMethod method, IObject obj) {
        this.method = method;
        this.obj = obj;
        setText();
        setImageDescriptor();
    }

    /**
     * sets the name for this action based on the method.
     */
    private void setText() {
        StringBuffer sb;
        try {
            sb = new StringBuffer(method.getName());
            sb.append('(');
            IClass[] types = method.getParameterTypes();
            for (int i = 0; i < types.length; i++) {
                sb.append(types[i].toString());
                if (i + 1 < types.length)
                    sb.append(", "); //$NON-NLS-1$
            }
            sb.append(')');
            setText(sb.toString());
        } catch (RemoteException e) {
            ObjBenchLogger.logError(e);
        }
    }

    /**
     * sets the image descriptor for this action based on the properties of the
     * method.
     */
    private void setImageDescriptor() {
        int adornments = 0;
        try {
            if (method.isAbstract())
                adornments |= JavaElementImageDescriptor.ABSTRACT;
            if (method.isStatic())
                adornments |= JavaElementImageDescriptor.STATIC;
            if (method.isFinal())
                adornments |= JavaElementImageDescriptor.FINAL;
            if (method.isSynchronized())
                adornments |= JavaElementImageDescriptor.SYNCHRONIZED;
        } catch (RemoteException re) {
            DetailsDialogUtil.openError(Display.getCurrent().getActiveShell(),
                    re);
            ObjBenchLogger.logError(re.getMessage(), re);
        }
        ImageDescriptor imgDesc = JavaUI.getSharedImages().getImageDescriptor(
                ISharedImages.IMG_OBJS_PUBLIC);
        setImageDescriptor(new JavaElementImageDescriptor(imgDesc, adornments,
                METHOD_ICON_SIZE));

    }

    public void run() {
        IObject[] arguments;
        IClass[] parameterTypes = method.getParameterTypes();
        if (parameterTypes.length == 0)
            arguments = new IObject[0];
        else {
            ExecuteMethodDialog dialog = new ExecuteMethodDialog(Display
                    .getCurrent().getActiveShell(), parameterTypes);
            switch (dialog.open()) {
            case Window.OK:
                try {
                    arguments = dialog.getArguments();
                } catch (ParserException pe) {
                    Throwable cause = pe.getCause();
                    if (cause instanceof RemoteException)
                        DetailsDialogUtil.openError(Display.getCurrent()
                                .getActiveShell(), cause);
                    else
                        MessageDialog.openError(null, ObjBenchPlugin
                                .getResourceString("Dialog.title.error"), pe //$NON-NLS-1$
                                .getLocalizedMessage());
                    return;
                } catch (ClassCastException cce) {
                    DetailsDialogUtil.openError(Display.getCurrent()
                            .getActiveShell(), cce);
                    return;
                }
                break;
            default:
                return;
            }
        }

        IObject returnedObj;
        try {
        	//TODO: added 11/10/05 - BM 
        	
        	// E-JUTR needs to be able to get at obj and arguments
        	
        	// get the object bench from the view, which we can find from the
            // workbench
        	
        	
        	ObjectBench bench = ObjBenchPlugin.getDefault().getBench();
        	bench.fireMethodCalledEvent(method, obj, arguments);

        	//TODO: end
        	
            returnedObj = method.invoke(obj, arguments);
            refreshPropertiesView();
            if (!method.getReturnType().getName().equals("void")) {
                MethodResultDialog mrd = new MethodResultDialog(Display
                        .getCurrent().getActiveShell(), method, returnedObj);
                mrd.open();
            } else {
                MessageDialog
                        .openInformation(
                                Display.getCurrent().getActiveShell(),
                                ObjBenchPlugin
                                        .getResourceString("Dialog.title"), ObjBenchPlugin.getResourceString("ExecuteMethodAction.successfulResult")); //$NON-NLS-1$ //$NON-NLS-2$
            }
        } catch (RemoteException re) {
            DetailsDialogUtil.openError(Display.getCurrent().getActiveShell(),
                    re);
            ObjBenchLogger.logError(re);
        }
    }

    private void refreshPropertiesView() {
        // refresh properties view, if it's visible
        IWorkbenchWindow activeWorkbenchWindow = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (null == activeWorkbenchWindow)
            return;
        IWorkbenchPage activePage = activeWorkbenchWindow.getActivePage();
        if (null == activePage)
            return;
        PropertiesView pView = (PropertiesView) activePage
                .findView(PropertiesView.class.getName());
        if (null == pView)
            return;
        pView.refresh();
    }
}