/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 26, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: RemoveInstanceAction.java,v 1.1 2006/01/23 16:07:15 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialogWithToggle;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.Instance;
import edu.wpi.ebob.internal.gui.dialogs.RemoveInstanceDialog;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.ObjectRemovalNotSupportedException;
import edu.wpi.ebob.preferences.OBPreferencePage;
import edu.wpi.ebob.views.ObjBenchView;

/**
 * Removes the instance currently selected in the Object Bench.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class RemoveInstanceAction extends Action implements
        ISelectionChangedListener {
    private final ImageDescriptor enabledImg = PlatformUI.getWorkbench()
            .getSharedImages()
            .getImageDescriptor(ISharedImages.IMG_TOOL_DELETE);

    private final ImageDescriptor disabledImg = PlatformUI.getWorkbench()
            .getSharedImages().getImageDescriptor(
                    ISharedImages.IMG_TOOL_DELETE_DISABLED);

    private Instance instance = null;

    private ISelectionProvider provider = null;

    /**
     * Constructs new <code>RemoveInstanceAction</code> using the
     * {@link ObjBenchView}'s selection provider. The ObjBenchView <b>MUST </B>
     * be on the workbench, or it won't be able to be retrieved; in this case,
     * an <code>IllegalStateException</code> is thrown.
     * 
     * @throws IllegalStateException
     *             if {@link ObjBenchView}can't be retrieved
     */
    public RemoveInstanceAction() {
        init();
        ObjBenchView view = (ObjBenchView) PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage().findView(
                        ObjBenchView.class.getName());
        if (null == view)
            throw new IllegalStateException(ObjBenchPlugin
                    .getResourceString("RemoveInstanceAction.viewMissing")); //$NON-NLS-1$
        provider = view.getSite().getSelectionProvider();
        provider.addSelectionChangedListener(this);

        // start off disabled
        setEnabled(false);
    }

    /**
     * Creates a new <code>RemoveInstanceAction</code> using the given
     * selection provider.
     * 
     * @param provider
     *            The selection provider that this instance will be registered
     *            with.
     */
    public RemoveInstanceAction(ISelectionProvider provider) {
        init();
        this.provider = provider;
        provider.addSelectionChangedListener(this);

        // start off disabled
        setEnabled(false);
    }

    /**
     * Creates a new <code>RemoveInstanceAction</code> for the purpose of
     * removing the specified instance.
     * 
     * @param instance
     */
    public RemoveInstanceAction(Instance instance) {
        init();
        this.instance = instance;

        // always enabled
        setEnabled(true);
    }

    /**
     * initializes this action.
     */
    private void init() {
        setText(ObjBenchPlugin.getResourceString("RemoveInstanceAction.text")); //$NON-NLS-1$
        setToolTipText(ObjBenchPlugin
                .getResourceString("RemoveInstanceAction.toolTip")); //$NON-NLS-1$
        setImageDescriptor(enabledImg);
        setDisabledImageDescriptor(disabledImg);
    }

    /**
     * Disposes this action by removing it as a listener from the selection
     * provider. This must be called by the creator of the action when the
     * action is no longer needed.
     */
    public void dispose() {
        if (null != provider)
            provider.removeSelectionChangedListener(this);
    }

    /**
     * Removes the selected instance from the bench.
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        if (null == instance) {
            ObjBenchLogger.logWarning(ObjBenchPlugin
                    .getResourceString("RemoveInstanceAction.instanceMissing")); //$NON-NLS-1$
            return;
        }

        // check for removal confirmation, if that pref is set
        IPreferenceStore store = ObjBenchPlugin.getDefault()
                .getPreferenceStore();
        if (store.getBoolean(OBPreferencePage.P_CONFIRM_REMOVE))
            if (MessageDialogWithToggle
                    .openOkCancelConfirm(
                            null,
                            ObjBenchPlugin.getResourceString("Dialog.title"), //$NON-NLS-1$
                            ObjBenchPlugin
                                    .getResourceString("RemoveInstanceAction.areYouSure"), //$NON-NLS-1$
                            ObjBenchPlugin
                                    .getResourceString("RemoveInstanceAction.rememberDecision"), //$NON-NLS-1$
                            false, store, OBPreferencePage.P_CONFIRM_REMOVE)
                    .getReturnCode() == Window.CANCEL)
                return;

        // get the object bench from the view, which we can find from the
        // workbench
        ObjectBench bench = ObjBenchPlugin.getDefault().getBench();
        try {
            bench.removeObject(instance.getInstanceName());
        } catch (ObjectRemovalNotSupportedException e) {
            RemoveInstanceDialog dialog = new RemoveInstanceDialog(Display
                    .getCurrent().getActiveShell(), instance);
            switch (dialog.open()) {
            case Window.CANCEL:
                return;
            case RemoveInstanceDialog.DELETE_FORCE:
                bench.removeObjectForced(instance.getInstanceName());
                break;
            case RemoveInstanceDialog.DELETE_CASCADE:
                bench.removeObjectCascade(instance.getInstanceName());
                break;
            default:
                assert false : dialog.getReturnCode();
            }
        }
    }

    /**
     * @see ISelectionChangedListener#selectionChanged(SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event) {
        IStructuredSelection selection = (IStructuredSelection) event
                .getSelection();
        if (selection.isEmpty()) {
            instance = null;
            setEnabled(false);
        } else {
            instance = (Instance) selection.getFirstElement();
            setEnabled(true);
        }
    }
}