/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 18, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *    Liam Morley - moved from edu.wpi.ebob.internal.dnd
 * 
 * $Id: TypeStatus.java,v 1.1 2006/01/23 16:08:41 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.viewers.IStructuredSelection;

import edu.wpi.ebob.ObjBenchPlugin;

/**
 * Represents the status of an eligible object for the Object Bench.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 * @see edu.wpi.ebob.internal.EBOBUtils
 *  
 */
public class TypeStatus extends Status {
    /** user has attempted to create multiple objects */
    public static final TypeStatus MULTIPLE_SELECTIONS = new TypeStatus(
            IStatus.ERROR, TypeStatus.CODE_MULTIPLE_SELECTIONS, ObjBenchPlugin
                    .getResourceString("DND.multiple.selections")); //$NON-NLS-1$

    /** user has attempted to create an object that is not a class */
    public static final TypeStatus NOT_CLASS = new TypeStatus(IStatus.ERROR,
            TypeStatus.CODE_NOT_CLASS, ObjBenchPlugin
                    .getResourceString("DND.not.class")); //$NON-NLS-1$

    /** user has attempted to create a class that is not public */
    public static final TypeStatus NOT_PUBLIC = new TypeStatus(IStatus.ERROR,
            TypeStatus.CODE_NOT_PUBLIC, ObjBenchPlugin
                    .getResourceString("DND.not.public")); //$NON-NLS-1$

    /** user has attempted to create an abstract class */
    public static final TypeStatus ABSTRACT = new TypeStatus(IStatus.ERROR,
            TypeStatus.CODE_ABSTRACT, ObjBenchPlugin
                    .getResourceString("DND.abstract")); //$NON-NLS-1$

    /** user has attempted to create multiple objects */
    public static final int CODE_MULTIPLE_SELECTIONS = 0x10;

    /** user has attempted to create an object that is not a type */
    public static final int CODE_NOT_TYPE = 0x20;

    /** user has attempted to create an object that is not a class */
    public static final int CODE_NOT_CLASS = 0x40;

    /** user has attempted to create a class that is not public */
    public static final int CODE_NOT_PUBLIC = 0x80;

    /** user has attempted to create an abstract class */
    public static final int CODE_ABSTRACT = 0x200;

    /** there was an exception thrown while getting the information */
    public static final int CODE_HAS_MODEL_PROBLEM = 0x400;

    /**
     * Uses the same parameters as {@link Status}. The Object Bench plug-in ID
     * is used, and <code>exception</code> is <code>null</code>.
     */
    public TypeStatus(int severity, int code, String message) {
        super(severity, ObjBenchPlugin.getDefault().getBundle()
                .getSymbolicName(), code, message, null);
    }

    /**
     * Uses the same parameters as {@link Status}. The Object Bench plug-in ID
     * is used, and <code>message</code> is set to the <code>exception</code>
     * 's localized message.
     */
    public TypeStatus(int severity, int code, Throwable exception) {
        super(severity, ObjBenchPlugin.getDefault().getBundle()
                .getSymbolicName(), code, exception.getLocalizedMessage(),
                exception);
    }

    /**
     * 
     * @param sel
     *            the selection
     * @return the acceptability status of the given selection
     */
    public static TypeStatus getStatus(IStructuredSelection sel) {
        // selection size must be 1
        if (sel.size() != 1)
            return TypeStatus.MULTIPLE_SELECTIONS;
    
        Object element = sel.getFirstElement();
        if (element instanceof IType)
            return getStatus((IType) element);
        else if (element instanceof ICompilationUnit) {
            return getStatus((ICompilationUnit) element);
        } else
            return TypeStatus.NOT_CLASS;
    }

    /**
     * @param unit
     *            the compilation unit
     * @return the acceptability status of the compilation unit's primary type
     */
    public static TypeStatus getStatus(ICompilationUnit unit) {
        return getStatus(unit.findPrimaryType());
    }

    /**
         * @param type
         *            the type
         * @return the acceptability status of the given type
         */
        public static TypeStatus getStatus(IType type) {
            if (null == type)
                return new TypeStatus(IStatus.ERROR, IStatus.ERROR, ObjBenchPlugin
                        .getResourceString("EBOBUtils.nullValue")); //$NON-NLS-1$
            try {
                if (!type.isClass())
                    return TypeStatus.NOT_CLASS;
                int f = type.getFlags();
                if (!Flags.isPublic(f))
                    return TypeStatus.NOT_PUBLIC;
                if (Flags.isAbstract(f))
                    return TypeStatus.ABSTRACT;
            } catch (JavaModelException jme) {
                return new TypeStatus(IStatus.ERROR,
                        TypeStatus.CODE_HAS_MODEL_PROBLEM, jme);
            }
            return new TypeStatus(IStatus.OK, IStatus.OK, ObjBenchPlugin
                    .getResourceString("messages.ok")); //$NON-NLS-1$
        }
}