/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Initially created on Dec 10, 2004
 * 
 * Contributors: Liam Morley - initial API and implementation
 * 
 * $Id: ObjBenchDropTargetListener.java,v 1.1 2006/01/23 16:08:45 bigben Exp $
 ******************************************************************************/
package edu.wpi.ebob.internal.dnd;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IType;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.navigator.LocalSelectionTransfer;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.actions.CreateInstanceAction;
import edu.wpi.ebob.internal.TypeStatus;

/**
 * Responds to users dragging classes and files over the Object Bench. If a Java
 * class is dropped on the Object Bench view, {@link CreateInstanceAction#run()}
 * is called.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 * @see CreateInstanceAction
 *  
 */
public final class ObjBenchDropTargetListener extends DropTargetAdapter {
    private static final LocalSelectionTransfer selectionTransfer = LocalSelectionTransfer
            .getInstance();

    private final Shell shell;

    /**
     * Creates a new instance, with a given shell to report errors to, and to
     * pass on to the wizard. All methods in this class are in
     * {@link org.eclipse.swt.dnd.DropTargetListener}.
     * 
     * @param shell
     *            the shell to report errors to, and pass on to the wizard
     */
    public ObjBenchDropTargetListener(final Shell shell) {
        this.shell = shell;
    }

    public void dragEnter(DropTargetEvent event) {
        if ((event.operations & DND.DROP_COPY) != 0)
            event.detail = DND.DROP_COPY;
        else
            event.detail = DND.DROP_NONE;
        for (int i = 0; i < event.dataTypes.length; i++)
            if (selectionTransfer.isSupportedType(event.dataTypes[i])) {
                event.currentDataType = event.dataTypes[i];
                break;
            }
    }

    public void dragOver(DropTargetEvent event) {
        TypeStatus status;
        Object obj = selectionTransfer.nativeToJava(event.currentDataType);
        // some systems don't support 'nativeToJava', we should let it go
        if (null == obj)
            return;

        status = TypeStatus.getStatus((IStructuredSelection) obj);
        if (!status.isOK())
            event.detail = DND.DROP_NONE;
    }

    public void dragOperationChanged(DropTargetEvent event) {
        if ((event.operations & DND.DROP_COPY) != 0)
            event.detail = DND.DROP_COPY;
        else
            event.detail = DND.DROP_NONE;
    }

    public void drop(DropTargetEvent event) {
        IStructuredSelection sel = (IStructuredSelection) selectionTransfer
                .getSelection();
        TypeStatus status = TypeStatus.getStatus(sel);

        if (!status.isOK()) {
            event.detail = DND.DROP_NONE;
            MessageDialog.openError(shell, ObjBenchPlugin
                    .getResourceString("Dialog.title"), status.getMessage()); //$NON-NLS-1$
            return;
        }

        // run the CreateInstanceAction
        CreateInstanceAction action = new CreateInstanceAction(shell,
                getType(sel));
        action.run();
    }

    /**
     * Retrieves the compilation unit
     * 
     * @param sel
     *            the selection
     * @return the compilation unit within the selection
     */
    private IType getType(IStructuredSelection sel) {
        if (sel.size() != 1)
            throw new IllegalArgumentException(ObjBenchPlugin
                    .getResourceString("DND.multiple.selections")); //$NON-NLS-1$
        Object element = sel.getFirstElement();
        if (element instanceof IType)
            return (IType) element;
        else if (element instanceof ICompilationUnit)
            return ((ICompilationUnit) element).findPrimaryType();
        else
            throw new IllegalArgumentException(ObjBenchPlugin
                    .getResourceString("DND.not.class")); //$NON-NLS-1$
    }
}