/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 11, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: Instance.java,v 1.1 2006/01/23 16:08:29 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui;

import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Comparator;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.IPropertySource;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.actions.ExecuteMethodAction;
import edu.wpi.ebob.actions.InspectInstanceAction;
import edu.wpi.ebob.actions.RemoveInstanceAction;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IMethod;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.preferences.OBPreferencePage;

/**
 * Represents an instance in the Object Bench. This keeps track of the actual
 * object itself as well as the name of the object.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public class Instance extends Composite implements IAdaptable {

    /**
     * Sorts two methods by name. For use with the context menu.
     * 
     * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
     *  
     */
    protected class MethodComparator implements Comparator {
        /**
         * @see Comparator#compare(Object, Object)
         * @see String#compareTo(String)
         */
        public int compare(Object o1, Object o2) {
            if (null == o1 || null == o2 || (o1 == o2)
                    || !(o1 instanceof IMethod) || !(o2 instanceof IMethod))
                return 0;
            IMethod m1 = (IMethod) o1, m2 = (IMethod) o2;
            int compareresult = 0; 
            try {
                compareresult = m1.getName().compareTo(m2.getName());
            } catch (RemoteException e) {
                ObjBenchLogger.logError(e);
            }
            return compareresult;
        }
    }

    /** size of instance */
    protected static final int DEFAULT_WIDTH = 120, DEFAULT_HEIGHT = 100;

    /**
     * the foreground color for this instance. This is only kept so that it can
     * be disposed of when the object is disposed of.
     */
    private Color fgColor;

    /**
     * the background color for all deselected instances.
     */
    private Color bgColor;

    /**
     * the background color for the one selected instance, if such an instance
     * is selected.
     */
    private Color bgColorSelected;

    /**
     * the current color of this Instance, either {@link #bgColor}or
     * {@link #bgColorSelected}.
     */
    private Color bgColorActive;

    /** displays what <code>Class</code> the object is. */
    private Label classLabel;

    /** the font that all labels will be given. */
    private Font font;

    /** the name of the object. */
    private String name;

    /** displays the name of the object. */
    private Label nameLabel;

    /** the object to keep track of. */
    private IObject obj;

    private Preferences.IPropertyChangeListener prefChangeListener;

    /**
     * Creates a new instance using <code>obj</code> as the underlying object
     * and <code>name</code> as its name.
     * 
     * @param name
     *            The name given to the object.
     * @param obj
     *            The object represented by this instance.
     */
    public Instance(Composite parent, String name, IObject obj) {
        super(parent, SWT.NONE);
        // validate name as possible variable
        IStatus validIdentifier = JavaConventions.validateIdentifier(name);
        if (!validIdentifier.isOK()) {
            this.dispose();
            throw new IllegalArgumentException(validIdentifier.getMessage());
        }

        this.obj = obj;
        this.name = name;

        // set colors
        setBGColor();
        setSelectedBGColor();
        bgColorActive = bgColor;

        setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);

        nameLabel = new Label(this, SWT.NONE);
        nameLabel.setBackground(bgColorActive);
        nameLabel.setAlignment(SWT.CENTER);
        classLabel = new Label(this, SWT.NONE);
        classLabel.setBackground(bgColorActive);
        classLabel.setAlignment(SWT.CENTER);
        setFGColor();
        refreshLabelDetails();

        addPaintListener(new PaintListener() {
            public void paintControl(PaintEvent e) {
                Instance.this.paintControl(e);
            }
        });

        addDisposeListener(new DisposeListener() {
            public void widgetDisposed(DisposeEvent de) {
                Instance.this.widgetDisposed();
            }
        });

        prefChangeListener = new Preferences.IPropertyChangeListener() {
            public void propertyChange(Preferences.PropertyChangeEvent event) {
                changeProperty(event.getProperty());
            }
        };
        ObjBenchPlugin.getDefault().getPluginPreferences()
                .addPropertyChangeListener(prefChangeListener);

        addMouseListener(new MouseAdapter() {
            /**
             * calls the 'Inspect' action.
             * 
             * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(MouseEvent)
             */
            public void mouseDoubleClick(MouseEvent e) {
                Instance.this.inspect();
            }
        });
    }

    /**
     * Makes sure children widgets also add the listeners they need.
     * 
     * @see org.eclipse.swt.widgets.Widget#addListener(int, Listener)
     */
    public void addListener(int eventType, Listener listener) {
        super.addListener(eventType, listener);
        // these are the events that children shouldn't know about
        if (eventType == SWT.Paint || eventType == SWT.Dispose)
            return;
        Control[] children = this.getChildren();
        for (int i = 0; i < children.length; i++)
            children[i].addListener(eventType, listener);
    }

    /**
     * @see org.eclipse.swt.widgets.Control#computeSize(int, int)
     */
    public Point computeSize(int wHint, int hHint) {
        int width = (wHint != SWT.DEFAULT) ? wHint : getSize().x;
        int height = (hHint != SWT.DEFAULT) ? hHint : getSize().y;
        return new Point(width, height);
    }

    /**
     * @see org.eclipse.swt.widgets.Control#computeSize(int, int, boolean)
     */
    public Point computeSize(int wHint, int hHint, boolean changed) {
        // ignore 'changed'
        return this.computeSize(wHint, hHint);
    }

    /**
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class adapter) {
        if (adapter == IPropertySource.class)
            return new InstancePropertySource(this);
        return null;
    }

    /** @return the name of this instance. */
    public String getInstanceName() {
        return name;
    }

    /** @return Returns the real object that this instance represents. */
    public IObject getObject() {
        return obj;
    }

    /**
     * launches the properties view.
     */
    public void inspect() {
        final String propsViewID = "edu.wpi.ebob.views.PropertiesView"; //$NON-NLS-1$
        try {
            PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                    .getActivePage().showView(propsViewID);
        } catch (PartInitException pie) {
            ObjBenchLogger.logError(pie);
            MessageDialog.openError(null, ObjBenchPlugin
                    .getResourceString("Dialog.title.error"), //$NON-NLS-1$
                    ObjBenchPlugin
                            .getResourceString("Instance.noPropertiesView")); //$NON-NLS-1$
        }
    }

    /**
     * Handles drawing the background rectangle.
     * 
     * @param pe
     *            the paint event.
     */
    protected void paintControl(PaintEvent pe) {
        GC gc = pe.gc;
        gc.setBackground(bgColorActive);
        gc.setForeground(this.getForeground());

        // draw background rectangle and outline
        gc.drawRoundRectangle(0, 0, getSize().x - 5, getSize().y - 5, 8, 8);
        gc.fillRoundRectangle(1, 1, getSize().x - 6, getSize().y - 6, 4, 4);

        // draw line in between name and class
        org.eclipse.swt.graphics.Rectangle r = nameLabel.getBounds();
        int lineHeight = r.y + r.height + 2;
        gc.drawLine(0, lineHeight, getSize().x - 5, lineHeight);
    }

    /**
     * Sets the background color of the labels to the current background color.
     */
    private void refreshLabelColors() {
        nameLabel.setBackground(bgColorActive);
        classLabel.setBackground(bgColorActive);
    }

    /**
     * Sets the bounds and text for the labels, dependent on the size of the
     * control.
     */
    private void refreshLabelDetails() {
        // set font
        font = new Font(null, PreferenceConverter.getFontData(ObjBenchPlugin
                .getDefault().getPreferenceStore(), OBPreferencePage.P_FONT));
        nameLabel.setFont(font);
        classLabel.setFont(font);

        // compensate for margin
        final int width = getSize().x - 10;
        // get maximum height of the first label
        final int maxHeight = getSize().y / 2 - 3;

        // compute the height of the text
        Point lp = nameLabel.computeSize(width, SWT.DEFAULT, false);
        if (lp.y > maxHeight)
            lp.y = maxHeight;
        nameLabel.setBounds(5, 5, width, lp.y);

        // place classLabel in relation to nameLabel's placement
        lp = classLabel.computeSize(width, SWT.DEFAULT, false);
        classLabel.setBounds(5, nameLabel.getBounds().height
                + nameLabel.getBounds().y + 5, width, lp.y);

        // reset label text if necessary
        setLabelText(nameLabel, MessageFormat.format(ObjBenchPlugin
                .getResourceString("Instance.instanceNameLabelText"), //$NON-NLS-1$
                new Object[] { this.name }));
        setLabelText(classLabel, obj.getIClass().toString());
    }

    /**
     * Sets the name of this instance to <code>name</code>.
     * 
     * @param name
     *            The new name for this instance.
     */
    public void setInstanceName(String name) {
        if (this.name.equals(name))
            return;
        this.name = name;
        refreshLabelDetails();
    }

    /**
     * Sets the label's text to <code>text</code>, provided there is enough
     * horizontal space to accomodate it. Otherwise, the last few characters of
     * the label's text are replaced by an ellipsis and the label's tooltip text
     * is set to the full value of <code>text</code>.
     * 
     * @param label
     *            the label to set the text on.
     * @param text
     *            the desired text
     */
    protected void setLabelText(Label label, String text) {
        final int availableWidth = label.getSize().x;
        String newText = new String(text);
        GC gc = new GC(this);
        gc.setFont(font);
        StringBuffer sb = new StringBuffer(newText);
        // keep on removing the last character until it's short enough
        while (gc.textExtent(newText).x > availableWidth) {
            sb.setLength(sb.length() - 1);
            newText = MessageFormat.format(ObjBenchPlugin
                    .getResourceString("Instance.concatenatedLabel"), //$NON-NLS-1$
                    new Object[] { sb.toString() });
        }
        gc.dispose();
        label.setText(newText);
        if (newText.length() < text.length())
            label.setToolTipText(text);
    }

    /**
     * Sets this Instance's selected state to <code>isSelected</code>.
     * 
     * @param isSelected
     *            <code>true</code> if this Instance should be selected,
     *            <code>false</code> otherwise.
     */
    public void setSelected(boolean isSelected) {
        bgColorActive = (isSelected) ? bgColorSelected : bgColor;
        refreshLabelColors();
        this.redraw();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return MessageFormat.format(ObjBenchPlugin
                .getResourceString("Instance.toString"), //$NON-NLS-1$
                new Object[] { name, this.getSize(),
                        this.getBounds().toString(), obj });
    }

    /**
     * Handles disposing of the widget and anything in it that wouldn't
     * otherwise get disposed.
     */
    protected void widgetDisposed() {
        if (null != bgColor && !bgColor.isDisposed())
            bgColor.dispose();
        if (null != bgColorSelected && !bgColorSelected.isDisposed())
            bgColorSelected.dispose();
        if (null != fgColor && !fgColor.isDisposed())
            fgColor.dispose();
        if (null != font && !font.isDisposed())
            font.dispose();
        ObjBenchPlugin.getDefault().getPluginPreferences()
                .removePropertyChangeListener(prefChangeListener);
    }

    /**
     * Sets the foreground color to the preferred color.
     * 
     * @see OBPreferencePage#P_FG_COLOR
     */
    private void setFGColor() {
        IPreferenceStore store = ObjBenchPlugin.getDefault()
                .getPreferenceStore();
        RGB rgb = PreferenceConverter.getColor(store,
                OBPreferencePage.P_FG_COLOR);
        fgColor = new Color(null, rgb);
        setForeground(fgColor);
        nameLabel.setForeground(fgColor);
        classLabel.setForeground(fgColor);
    }

    /**
     * Sets the background color to the preferred color.
     * 
     * @see OBPreferencePage#P_BG_COLOR
     */
    private void setBGColor() {
        IPreferenceStore store = ObjBenchPlugin.getDefault()
                .getPreferenceStore();
        RGB rgb = PreferenceConverter.getColor(store,
                OBPreferencePage.P_BG_COLOR);
        bgColor = new Color(null, rgb);
    }

    /**
     * Sets the background color to the preferred color.
     * 
     * @see OBPreferencePage#P_BG_COLOR
     */
    private void setSelectedBGColor() {
        IPreferenceStore store = ObjBenchPlugin.getDefault()
                .getPreferenceStore();
        RGB rgb = PreferenceConverter.getColor(store,
                OBPreferencePage.P_BG_COLOR_SELECTED);
        bgColorSelected = new Color(null, rgb);
    }

    /**
     * A callback function that adds the instance-specific menu actions to the
     * <code>manager</code>. This also adds a Separator with
     * <code>IWorkbenchActionConstants.MB_ADDITIONS</code>.
     */
    public void menuAboutToShow(IMenuManager manager) {
        // add "Inspect" and "Remove"
        manager.add(new InspectInstanceAction(this));
        manager.add(new RemoveInstanceAction(this));

        manager.add(new Separator());

        // add methods for each parent class
        for (IClass c = obj.getIClass(); c != null; c = c.getSuperclass()) {
            IMethod[] methods = null;
            try {
                methods = c.getMethods();
            } catch (RemoteException re) {
                DetailsDialogUtil.openError(Display.getCurrent()
                        .getActiveShell(), re);
                ObjBenchLogger.logError(re.getMessage(), re);
            }
            // only add methods if there is something to add
            if (methods.length > 0) {
                Arrays.sort(methods, new MethodComparator());

                /*
                 * for each parent class, we want to make a submenu. We don't
                 * want to have to know whether we're adding to a submenu or the
                 * top-level, so we refer to the current menu as "mgr", which
                 * could be either the top-level "manager" menu or a submenu
                 * child of manager
                 */
                IMenuManager mgr;
                if (!c.equals(obj.getClass())) {
                    String cName;
                    try {
                        cName = c.getName();
                    } catch (RemoteException re) {
                        ObjBenchLogger.logError(re);
                        cName = c.objToString();
                    }
                    mgr = new MenuManager(cName);
                    manager.add(mgr);
                } else
                    mgr = manager;

                for (int i = 0; i < methods.length; i++) {
                    IMethod method = methods[i];
                    // only add methods that are from this class
                    if (!method.getDeclaringClass().equals(c))
                        continue;
                    mgr.add(new ExecuteMethodAction(method, obj));
                }
            }
        }

        manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
    }

    /**
     * sets menu for both this control as well as all controls which are
     * children of this control.
     * 
     * @see Control#setMenu(Menu)
     */
    public void setMenu(Menu menu) {
        super.setMenu(menu);
        Control[] children = getChildren();
        for (int i = 0; i < children.length; i++)
            children[i].setMenu(menu);
    }

    /**
     * changes a property of the instance based on a preference change.
     * 
     * @param property
     *            the property you wish to change, from {@link OBPreferencePage}.
     * @see OBPreferencePage
     */
    protected void changeProperty(String property) {
        if (OBPreferencePage.P_FONT.equals(property)) {
            // about to reassign the font- dispose of the current one
            font.dispose();
            refreshLabelDetails();
            // just in case a size change moved the line between the
            // labels, redraw the instance
            redraw();
        } else if (OBPreferencePage.P_FQ_CLASS_NAMES.equals(property)) {
            refreshLabelDetails();
        } else if (OBPreferencePage.P_FG_COLOR.equals(property)) {
            fgColor.dispose();
            setFGColor();
            redraw();
        } else if (OBPreferencePage.P_BG_COLOR.equals(property)) {
            bgColor.dispose();
            setBGColor();
            setSelected(bgColorActive == bgColorSelected);
        } else if (OBPreferencePage.P_BG_COLOR_SELECTED.equals(property)) {
            boolean isSelected = (bgColorActive == bgColorSelected);
            bgColorSelected.dispose();
            setSelectedBGColor();
            setSelected(isSelected);
        }
    }
}