/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 9, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: InstanceContainer.java,v 1.1 2006/01/23 16:08:29 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui;

import java.text.MessageFormat;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.views.navigator.LocalSelectionTransfer;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.dnd.ObjBenchDropTargetListener;
import edu.wpi.ebob.model.ObjectBenchEvent;
import edu.wpi.ebob.model.ObjectBenchListener;
import edu.wpi.ebob.model.MethodInvokeInfo;

/**
 * A graphical container for instances in the Object Bench. This listens to
 * changes in the Object Bench model.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class InstanceContainer implements ISelectionProvider,
        ObjectBenchListener {
    /** the panel which will contain all instances */
    private final Composite panel;

    /** the scrolling container which contains <code>panel</code>. */
    private final ScrolledComposite sc;

    /** a list of all selection listeners */
    private final ListenerList selectionChangedListeners = new ListenerList(3);

    /** a mouse listener for handling selection */
    private final ICMouseListener mListener;

    /** the currently selected instance in the container */
    private Instance selectedInstance;

    /**
     * The menu manager for this container. In charge of instance children as
     * well.
     */
    private MenuManager menuMgr = new MenuManager();

    /**
     * constructs a new container with a given parent.
     * 
     * @param parent
     *            the parent composite that this container will live inside of.
     */
    public InstanceContainer(Composite parent) {
        sc = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.BORDER);
        sc.setExpandHorizontal(true);
        sc.setExpandVertical(true);

        mListener = new ICMouseListener(this);

        panel = new Composite(sc, SWT.NONE) {
            // keeps the same width as the parent so that instances always wrap
            public Point computeSize(int wHint, int hHint, boolean changed) {
                return super.computeSize(getParent().getClientArea().width,
                        SWT.DEFAULT, changed);
            }
        };
        panel.setLayout(new RowLayout());
        panel.addMouseListener(mListener);
        sc.setContent(panel);
        panel.setLayout(new RowLayout());
        // keep the size updated as things get resized
        sc.addControlListener(new ControlAdapter() {
            public void controlResized(ControlEvent e) {
                sc.setMinSize(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT));
            }
        });

        menuMgr.setRemoveAllWhenShown(true);
        hookDnD();
    }

    /**
     * @see ISelectionProvider#addSelectionChangedListener(ISelectionChangedListener)
     */
    public void addSelectionChangedListener(ISelectionChangedListener listener) {
        selectionChangedListeners.add(listener);
    }

    /**
     * alerts all listeners to any changes.
     * 
     * @param event
     *            an event that listeners should be aware of.
     */
    private void fireSelectionChanged(final SelectionChangedEvent event) {
        Object[] listeners = selectionChangedListeners.getListeners();
        for (int i = 0; i < listeners.length; ++i) {
            final ISelectionChangedListener l = (ISelectionChangedListener) listeners[i];
            Platform.run(new SafeRunnable() {
                public void run() {
                    l.selectionChanged(event);
                }
            });
        }
    }

    /**
     * retrieves the context menu for the purposes of display, based on the
     * items supplied.
     * 
     * @return the context menu for the container.
     */
    public MenuManager getContextMenu(final IAction[] items) {
        menuMgr.addMenuListener(new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                if (null != selectedInstance)
                    selectedInstance.menuAboutToShow(manager);
                else
                    for (int i = 0; i < items.length; i++) {
                        manager.add(items[i]);
                        manager.add(new Separator(
                                IWorkbenchActionConstants.MB_ADDITIONS));
                    }
            }
        });

        Menu menu = menuMgr.createContextMenu(panel);
        panel.setMenu(menu);
        return menuMgr;
    }

    /**
     * @see ISelectionProvider#getSelection()
     */
    public ISelection getSelection() {
        if (null != selectedInstance)
            return new StructuredSelection(selectedInstance);
        else
            return StructuredSelection.EMPTY;
    }

    /** facilitates drag and drop functionality. */
    private void hookDnD() {
        // accept any operation as long as it's COPY
        final DropTarget target = new DropTarget(panel, DND.DROP_COPY
                | DND.DROP_MOVE | DND.DROP_LINK | DND.DROP_DEFAULT);

        target.setTransfer(new Transfer[] { LocalSelectionTransfer
                .getInstance() });
        target
                .addDropListener(new ObjBenchDropTargetListener(panel
                        .getShell()));
    }

    /**
     * @see ObjectBenchListener#objectAdded(ObjectBenchEvent)
     */
    public void objectAdded(ObjectBenchEvent event) {
        Instance instance = new Instance(panel, event.getName(), event
                .getObject());
        instance.addMouseListener(mListener);
        instance.setMenu(menuMgr.createContextMenu(instance));
        final Point computeSize = panel.computeSize(SWT.DEFAULT, SWT.DEFAULT);
        panel.layout();
        sc.setMinSize(computeSize);
    }

    /**
     * @see ObjectBenchListener#objectRemoved(ObjectBenchEvent)
     */
    public void objectRemoved(ObjectBenchEvent event) {
        String name = event.getName();
        // look through children to see if we find it
        Control[] children = panel.getChildren();
        for (int i = 0; i < children.length; i++) {
            Instance instance = (Instance) children[i];
            if (name.equals(instance.getInstanceName())) {
                // we all share the same menu- don't dispose it
                instance.setMenu(null);
                instance.dispose();
                sc.setMinSize(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT));
                panel.layout();
                // check to see if this is the current selection
                if (instance == selectedInstance)
                    setSelection(StructuredSelection.EMPTY);
                return;
            }
        }
        // if we get here, that means the object was never found; that's bad.
        ObjBenchLogger.logWarning(MessageFormat.format(ObjBenchPlugin
                .getResourceString("InstanceContainer.unknownInstanceRemoved"), //$NON-NLS-1$
                new Object[] { name }));
    }

    //TODO: 11/10/05 - BM Edit - changes to ObjectBenchListener necessitated the 
    // 		                     addition of this method.    
    public void methodCalled(MethodInvokeInfo mii) {
    	//fortunately this method will never be called within EBOB so I don't think we have to worry about it.
    }   
    //TODO: 11/10/05 - end
    /**
     * @see ISelectionProvider#removeSelectionChangedListener(ISelectionChangedListener)
     */
    public void removeSelectionChangedListener(
            ISelectionChangedListener listener) {
        selectionChangedListeners.remove(listener);
    }

    /** gives the container focus. */
    public void setFocus() {
        panel.setFocus();
    }

    /**
     * @see ISelectionProvider#setSelection(ISelection)
     */
    public void setSelection(ISelection selection) {
        Instance newInstance = (Instance) ((StructuredSelection) selection)
                .getFirstElement();

        if (newInstance != selectedInstance) {
            if (null != selectedInstance && !selectedInstance.isDisposed())
                selectedInstance.setSelected(false);
            selectedInstance = newInstance;
            if (null != selectedInstance && !selectedInstance.isDisposed())
                selectedInstance.setSelected(true);
            fireSelectionChanged(new SelectionChangedEvent(this, selection));
        }
    }
}

/**
 * Listens to mouseDown events for the purpose of registering selection.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */

class ICMouseListener extends MouseAdapter {
    private InstanceContainer container;

    /** the last time mouseDown was fired */
    private int lastTime;

    private static final int DEFAULT_LAST_TIME = -1;

    ICMouseListener(InstanceContainer container) {
        this.container = container;
        lastTime = DEFAULT_LAST_TIME;
    }

    public void mouseDown(MouseEvent e) {
        // make sure this wasn't a double-click
        int doubleClickTime = e.display.getDoubleClickTime();
        if (doubleClickTime != 0) {
            if (lastTime == DEFAULT_LAST_TIME)
                lastTime = e.time;
            else {
                int lt = lastTime;
                lastTime = DEFAULT_LAST_TIME;
                if (doubleClickTime >= e.time - lt)
                    return;
            }
        }

        // this shouldn't happen, but just in case
        if (!(e.widget instanceof Control)) {
            ObjBenchLogger.logWarning(ObjBenchPlugin
                    .getResourceString("InstanceContainer.unknownWidget")); //$NON-NLS-1$
            container.setSelection(StructuredSelection.EMPTY);
            return;
        }

        Instance instance = getInstance((Control) e.widget);
        // if we've selected an instance, then set the selection
        if (null != instance)
            container.setSelection(new StructuredSelection(instance));
        // otherwise set the selection to EMPTY
        else
            container.setSelection(StructuredSelection.EMPTY);
    }

    /**
     * obtains the Instance that contains this control if the control itself is
     * an Instance, or if a parent of the control is an Instance.
     * 
     * @param source
     *            the Control for which you desire an Instance.
     * @return the Instance that contains this control, or <code>null</code>
     *         if no such Instance contains this control.
     */
    private Instance getInstance(Control source) {
        while (null != source && !source.getClass().equals(Instance.class))
            source = source.getParent();
        return (Instance) source;
    }
}