/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 9, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: InstancePropertySource.java,v 1.1 2006/01/23 16:08:29 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.cm.IObject;

/**
 * A handler for instance property data. Passed to the Properties view for
 * displaying an instance's properties.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class InstancePropertySource implements IPropertySource {
    /** the instance whose properties we are concerned about. */
    private final Instance instance;

    /** a reference directly to the instance's name. */
    private final String instanceName;

    /** a reference directly to the instance's object. */
    private final IObject instanceObj;

    /** these are cached and returned upon subsequent calls. */
    private IPropertyDescriptor[] propertyDescriptors;

    /**
     * constructs a new property source for the given instance.
     * 
     * @param instance
     *            the instance whose properties we are concerned with.
     */
    public InstancePropertySource(Instance instance) {
        this.instance = instance;
        instanceObj = instance.getObject();
        instanceName = instance.getInstanceName();
    }

    /**
     * @see IPropertySource#getEditableValue()
     */
    public Object getEditableValue() {
        return instance;
    }

    /**
     * @see IPropertySource#getPropertyDescriptors()
     */
    public IPropertyDescriptor[] getPropertyDescriptors() {
        if (propertyDescriptors == null) {
            List descriptorList = new ArrayList();
            PropertyDescriptor name = new TextPropertyDescriptor(instanceName,
                    ObjBenchPlugin
                            .getResourceString("InstancePropertySource.name")); //$NON-NLS-1$
            name
                    .setCategory(ObjBenchPlugin
                            .getResourceString("InstancePropertySource.generalCategory")); //$NON-NLS-1$
            PropertyDescriptor clazz = new TextPropertyDescriptor(instanceObj
                    .getIClass(), ObjBenchPlugin
                    .getResourceString("InstancePropertySource.class")); //$NON-NLS-1$
            clazz
                    .setCategory(ObjBenchPlugin
                            .getResourceString("InstancePropertySource.generalCategory")); //$NON-NLS-1$
            descriptorList.add(name);
            descriptorList.add(clazz);

            // dependencies
            ObjectBench bench = ObjBenchPlugin.getDefault().getBench();
            String[] dependents = bench.getDependents(instanceName);
            for (int i = 0; i < dependents.length; i++) {
                PropertyDescriptor dependent = new TextPropertyDescriptor(
                        dependents[i],
                        ObjBenchPlugin
                                .getResourceString("InstancePropertySource.hasDependency")); //$NON-NLS-1$
                dependent
                        .setCategory(ObjBenchPlugin
                                .getResourceString("InstancePropertySource.dependencyCategory")); //$NON-NLS-1$
                descriptorList.add(dependent);
            }

            propertyDescriptors = new IPropertyDescriptor[descriptorList.size()];
            for (int i = 0; i < descriptorList.size(); i++)
                propertyDescriptors[i] = (IPropertyDescriptor) descriptorList
                        .get(i);
        }
        return propertyDescriptors;
    }

    /**
     * @see IPropertySource#getPropertyValue(Object)
     */
    public Object getPropertyValue(Object id) {
        return id.toString();
    }

    /**
     * There is no default value.
     * 
     * @see IPropertySource#isPropertySet(Object)
     */
    public boolean isPropertySet(Object id) {
        return true;
    }

    /**
     * Does nothing.
     * 
     * @see IPropertySource#resetPropertyValue(Object)
     */
    public void resetPropertyValue(Object id) {
        // does nothing
    }

    /**
     * @see IPropertySource#setPropertyValue(Object, Object)
     */
    public void setPropertyValue(Object id, Object value) {
        ObjBenchLogger.logWarning(ObjBenchPlugin
                .getResourceString("InstancePropertySource.unknownPropSet")); //$NON-NLS-1$
    }
}