/*******************************************************************************
 * Copyright (c) 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Mar 2, 2005
 *
 * Contributors:
 *    Justin Braga - initial API and implementation
 * 
 * $Id: ExecuteMethodDialog.java,v 1.1 2006/01/23 16:08:04 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.utils.ParameterChangeListener;
import edu.wpi.ebob.internal.gui.utils.ParameterTableViewer;
import edu.wpi.ebob.model.Parameter;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.parser.ParserException;

/**
 * JFace dialog for handling method invocation
 * 
 * @author <a href="mailto:trevelyan2001@users.sourceforge.net">Justin Braga
 *         </a>
 *  
 */
public class ExecuteMethodDialog extends Dialog implements
        ParameterChangeListener {

    private TableViewer tableViewer;

    private IClass[] parameterTypes;

    private boolean okState;

    private String statusMessage;

    /**
     * @param parentShell
     *            The parent of the dialog box
     * @param parameterTypes
     *            The parameter types for the selected method
     */
    public ExecuteMethodDialog(Shell parentShell, IClass[] parameterTypes) {
        super(parentShell);
        this.parameterTypes = parameterTypes;
        okState = false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);

        // The Instruction
        Label instruction = new Label(composite, SWT.WRAP);
        instruction.setText(ObjBenchPlugin
                .getResourceString("ExecuteMethodDialog.instructionLabel")); //$NON-NLS-1$
        GridData data = new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER);
        data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
        instruction.setLayoutData(data);
        instruction.setFont(parent.getFont());

        // The type/argument table
        ParameterTableViewer ptv = new ParameterTableViewer(composite);
        ptv.addParameterChangeListener(this);
        tableViewer = ptv.getTableViewer();
        tableViewer.setInput(ParameterTableViewer.toParameters(parameterTypes));

        return composite;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(ObjBenchPlugin
                .getResourceString("ExecuteMethodDialog.title")); //$NON-NLS-1$
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
        if (statusMessage == null && okState) {
            super.okPressed();
        } else {
            MessageDialog
                    .openError(
                            Display.getCurrent().getActiveShell(),
                            ObjBenchPlugin
                                    .getResourceString("Dialog.title.error"), statusMessage); //$NON-NLS-1$
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see edu.wpi.ebob.internal.gui.utils.ParameterChangeListener#parameterChanged()
     */
    public void parameterChanged() {
        Parameter[] args = (Parameter[]) tableViewer.getInput();
        if (null != args) {
            boolean argsAreIncomplete = false;
            for (int i = 0; i < args.length; i++) {
                Parameter arg = args[i];
                try {
                    // don't update status if we have an incomplete argument
                    if (null == arg.getInput())
                        argsAreIncomplete = true;
                    else
                        arg.evaluate();
                } catch (ParserException pe) {
                    okState = false;
                    statusMessage = (pe.getCause() instanceof RemoteException) ? pe
                            .getCause().getLocalizedMessage()
                            : pe.getLocalizedMessage();
                    return;
                } catch (ClassCastException cce) {
                    okState = false;
                    statusMessage = ObjBenchPlugin
                            .getResourceString("ExecuteMethodDialog.classCastException"); //$NON-NLS-1$
                    return;
                }
            }
            if (!argsAreIncomplete) {
                okState = true;
                statusMessage = null;
            }
        }
    }

    public IObject[] getArguments() throws ParserException, ClassCastException {
        Parameter[] args = (Parameter[]) tableViewer.getInput();
        IObject[] returnedArgs = new IObject[args.length];
        for (int i = 0; i < args.length; i++)
            returnedArgs[i] = args[i].evaluate();
        return returnedArgs;
    }
}
