/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 26, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: ArrayItemNode.java,v 1.1 2006/01/23 16:05:49 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import java.text.MessageFormat;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.model.cm.IArray;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IField;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class ArrayItemNode extends AbstractNode implements ModifiableNode {

    private boolean hasCheckedForChildren;

    private int index;

    private IClass type;

    private IObject value;

    private IArray array;

    public ArrayItemNode(IArray valArray, int index, IClass type) {
        this.array = valArray;
        this.index = index;
        this.type = type;
        try {
            this.value = valArray.get(index);
        } catch (RemoteException cme) {
            // not much you can do
            ObjBenchLogger.logError(cme);
        }
    }

    /**
     * @see Node#getChildren()
     */
    public Node[] getChildren() {
        if (!hasCheckedForChildren) {
            hasCheckedForChildren = true;
            if (null != value) {
                if (value.getIClass().isArray()) {
                    IArray valArray = (IArray) value;
                    for (int i = 0; i < valArray.getLength(); i++)
                        addChild(new ArrayItemNode(valArray, i, type));
                } else if (!type.getComponentType().isPrimitive()) {
                    IField[] fields = null;
                    try {
                        fields = value.getIClass().getDeclaredFields();
                    } catch (RemoteException re) {
                        DetailsDialogUtil.openError(Display.getCurrent()
                                .getActiveShell(), re);
                        ObjBenchLogger.logError(re.getMessage(), re);
                    }
                    for (int i = 0; i < fields.length; i++)
                        addChild(new FieldNode(value, fields[i]));
                }
            }
        }
        return super.getChildren();
    }

    /**
     * @return the parent node's icon
     * @see edu.wpi.ebob.internal.inspect.Node#getImage()
     */
    public Image getImage() {
        return getParent().getImage();
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getName()
     */
    public String getName() {
        try {
            return MessageFormat.format(ObjBenchPlugin
                    .getResourceString("ArrayItemNode.valueAsString"), //$NON-NLS-1$
                    new Object[] { type.getComponentType().getName(),
                            Integer.toString(index) });
        } catch (RemoteException re) {
            return MessageFormat
                    .format(
                            ObjBenchPlugin
                                    .getResourceString("ArrayItemNode.errorValueAsString"), new Object[] { Integer //$NON-NLS-1$
                                    .toString(index) });
        }
    }

    /**
     * @see Node#getType()
     */
    public IClass getType() {
        return type;
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getValue()
     */
    public IObject getValue() {
        return value;
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getValueAsString()
     */
    public String getValueAsString() {
        if (null == getValue())
            return "null"; //$NON-NLS-1$
        else if (value.getIClass().isArray()) {
            IArray valArray = (IArray) value;
            try {
                return MessageFormat.format(ObjBenchPlugin
                        .getResourceString("ArrayItemNode.name"), new Object[] { //$NON-NLS-1$
                        type.getComponentType().getName(),
                                Integer.toString(valArray.getLength()) });
            } catch (RemoteException re) {
                return MessageFormat
                        .format(
                                ObjBenchPlugin
                                        .getResourceString("ArrayItemNode.errorName"), new Object[] { Integer //$NON-NLS-1$
                                        .toString(valArray.getLength()) });
            }
        } else if (type.getComponentType().isPrimitive())
            return value.toString();
        else
            try {
                if (type.getComponentType().equals(
                        ObjBenchPlugin.getDefault().getClassManager().getClass(
                                "java.lang.String"))) //$NON-NLS-1$
                    return '"' + value.toString() + '"';
            } catch (ClassNotFoundException cnfe) {
                // won't happen
            }
        return value.getClass().getName();
    }

    /**
     * @return <code>true</code> if the object is non-null and any of the
     *         following are true:
     *         <ol>
     *         <li>the object is an array with a length greater than 0
     *         <li>the object is not-primitive and has at least one declared
     *         field
     *         </ol>
     * @see Node#hasChildren()
     */
    public boolean hasChildren() {
        if (null == value)
            return false;
        else if (value.getIClass().isArray()
                && ((IArray) value).getLength() > 0)
            return true;
        else
            try {
                return (!type.getComponentType().isPrimitive() && value
                        .getIClass().getDeclaredFields().length > 0);
            } catch (RemoteException re) {
                // whatcha gonna do?
                return false;
            }
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.ModifiableNode#isModifiable()
     */
    public boolean isModifiable() {
        Node parent = getParent();
        if (parent instanceof ModifiableNode)
            return ((ModifiableNode) parent).isModifiable();
        else
            return false;
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.ModifiableNode#setValue(IObject)
     */
    public void setValue(IObject newValue) {
        try {
            array.set(newValue, index);
            // reset cache
            hasCheckedForChildren = false;
            children.clear();
        } catch (RemoteException re) {
            // eat it
            ObjBenchLogger.logError(re);
        }
    }
}