/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html Initially created on
 * Dec 19, 2004 Contributors: Liam Morley - initial API and implementation $Id:
 * FieldNode.java,v 1.13 2005/03/12 02:48:54 imotic Exp $
 ******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import java.text.MessageFormat;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaElementImageDescriptor;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.model.cm.IArray;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IField;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * represents a {@link IField}in a tree.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public class FieldNode extends AbstractNode implements ModifiableNode {

    /** the field to display in the tree. */
    private IField field;

    /** we only want to check for children once. */
    private boolean hasCheckedForChildren = false;

    /** we only want to retrieve the value once. */
    private boolean hasRetrievedValue = false;

    /** the object that this field belongs to. */
    private IObject obj;

    /** the value of the field {@link #field}in object {@link #obj}. */
    private IObject value;

    /**
     * creates a new node in the tree representing a field.
     * 
     * @param obj
     *            the object that this field belongs to.
     * @param field
     *            the field to display in the tree.
     */
    public FieldNode(IObject obj, IField field) {
        this.obj = obj;
        this.field = field;
    }

    /**
     * @see Node#getChildren()
     */
    public Node[] getChildren() {
        if (!hasCheckedForChildren) {
            hasCheckedForChildren = true;
            if (null != getValue()) {
                if (field.getType().isArray()) {
                    IArray valArray = (IArray) value;
                    for (int i = 0; i < valArray.getLength(); i++)
                        addChild(new ArrayItemNode(valArray, i, field.getType()));
                } else if (!field.getType().isPrimitive()) {
                    IField[] fields = null;
                    try {
                        fields = value.getIClass().getDeclaredFields();
                    } catch (RemoteException re) {
                        DetailsDialogUtil.openError(Display.getCurrent()
                                .getActiveShell(), re);
                        ObjBenchLogger.logError(re.getMessage(), re);
                    }
                    for (int i = 0; i < fields.length; i++)
                        addChild(new FieldNode(value, fields[i]));
                }
            }
        }
        return super.getChildren();
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getImage()
     */
    public Image getImage() {
        String visibility = null;
        int adornments = 0;
        try {
            if (field.isPublic())
                //visibility = ISharedImages.IMG_FIELD_PUBLIC;
                visibility = ISharedImages.IMG_OBJS_PUBLIC;
            else if (field.isProtected())
                //visibility = ISharedImages.IMG_FIELD_PROTECTED;
                visibility = ISharedImages.IMG_OBJS_PROTECTED;
            else if (field.isPrivate())
                //visibility = ISharedImages.IMG_FIELD_PUBLIC;
                visibility = ISharedImages.IMG_OBJS_PRIVATE;
            else
                //visibility = ISharedImages.IMG_FIELD_PUBLIC;
                visibility = ISharedImages.IMG_OBJS_DEFAULT;
            // now add decorators
            if (field.isStatic())
                adornments |= JavaElementImageDescriptor.STATIC;
            if (field.isFinal())
                adornments |= JavaElementImageDescriptor.FINAL;
            if (field.isSynchronized())
                adornments |= JavaElementImageDescriptor.SYNCHRONIZED;
        } catch (RemoteException re) {
            DetailsDialogUtil.openError(Display.getCurrent().getActiveShell(),
                    re);
            ObjBenchLogger.logError(re.getMessage(), re);
        }
        // put it all together
        ImageDescriptor imgDesc = JavaUI.getSharedImages().getImageDescriptor(
                visibility);
        JavaElementImageDescriptor jImgDesc = new JavaElementImageDescriptor(
                imgDesc, adornments, new Point(16, 16));
        return jImgDesc.createImage();

    }

    /**
     * @see Node#getName()
     */
    public String getName() {
        try {
            return field.getName();
        } catch (RemoteException re) {
            ObjBenchLogger.logError(re);
            return ObjBenchPlugin.getResourceString("FieldNode.field"); //$NON-NLS-1$
        }
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getType()
     */
    public IClass getType() {
        return field.getType();
    }

    /**
     * @see Node#getValue()
     */
    public IObject getValue() {
        if (!hasRetrievedValue) {
            hasRetrievedValue = true;
            try {
                value = field.get(obj);
            } catch (IllegalArgumentException iae) {
                ObjBenchLogger.logError(iae);
            } catch (RemoteException re) {
                DetailsDialogUtil.openError(Display.getCurrent()
                        .getActiveShell(), re);
                ObjBenchLogger.logError(re.getMessage(), re);
            }
        }
        return value;
    }

    /**
     * @return an empty string if the field type is not primitive.
     * @see Node#getValueAsString()
     */
    public String getValueAsString() {
        // only return value for a primitive type, when we can retrieve it
        if (null == getValue())
            return "null"; //$NON-NLS-1$
        IClass type = field.getType();
        if (type.isArray()) {
            IArray valArray = ((IArray) value);
            try {
                return MessageFormat
                        .format(
                                ObjBenchPlugin
                                        .getResourceString("FieldNode.valueAsString"), new Object[] { //$NON-NLS-1$
                                type.getComponentType().getName(),
                                        Integer.toString(valArray.getLength()) });
            } catch (RemoteException re) {
                return MessageFormat
                        .format(
                                ObjBenchPlugin
                                        .getResourceString("FieldNode.errorValueAsString"), new Object[] { Integer //$NON-NLS-1$
                                        .toString(valArray.getLength()) });
            }
        } else if (type.isPrimitive())
            return value.toString();
        else
            try {
                if (type.equals(ObjBenchPlugin.getDefault().getClassManager()
                        .getClass("java.lang.String"))) //$NON-NLS-1$
                    return '"' + value.toString() + '"';
                try {
                    return value.getIClass().getName();
                } catch (RemoteException re) {
                    ObjBenchLogger.logError(re);
                    return ObjBenchPlugin.getResourceString("FieldNode.type"); //$NON-NLS-1$
                }
            } catch (ClassNotFoundException cnfe) {
                // this should never happen, log as an error and return junk
                ObjBenchLogger.logError(cnfe);
                return ObjBenchPlugin.getResourceString("FieldNode.type"); //$NON-NLS-1$
            }
    }

    /**
     * @return <code>true</code> if the object is non-null and any of the
     *         following are true:
     *         <ol>
     *         <li>the object is an array with a length greater than 0
     *         <li>the object is not-primitive and has at least one declared
     *         field
     *         </ol>
     * @see Node#hasChildren()
     */
    public boolean hasChildren() {
        if (null == getValue())
            return false;
        else if (field.getType().isArray() && ((IArray) value).getLength() > 0)
            return true;
        else
            try {
                return (!field.getType().isPrimitive() && value.getIClass()
                        .getDeclaredFields().length > 0);
            } catch (RemoteException re) {
                ObjBenchLogger.logError(re);
                return false;
            }
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.ModifiableNode#isModifiable()
     */
    public boolean isModifiable() {
        try {
            return field.isPublic() && !field.isFinal();
        } catch (RemoteException re) {
            ObjBenchLogger.logError(re.getMessage(), re);
            return false;
        }
    }

    /**
     * sets the value of the field. throws an exception if the field is not
     * public.
     * 
     * @param newValue
     *            the new value for this field.
     * @see IField
     */
    public void setValue(IObject newValue) {
        try {
            field.set(obj, newValue);
            // reset cache
            hasRetrievedValue = false;
            hasCheckedForChildren = false;
            children.clear();
        } catch (RemoteException re) {
            DetailsDialogUtil.openError(Display.getCurrent().getActiveShell(),
                    re);
            ObjBenchLogger.logError(re);
        }
    }
}