/*******************************************************************************
 * Copyright (c) 2005 Worcester Polytechnic Institute. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Initially created on Mar 17, 2005
 * 
 * Contributors: Liam Morley - initial API and implementation
 * 
 * $Id: ReturnValueNode.java,v 1.1 2006/01/23 16:05:49 bigben Exp $
 ******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import java.text.MessageFormat;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.model.cm.IArray;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IField;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * A Return value node represents an object that is returned from a computation,
 * which can be unnamed.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public class ReturnValueNode extends AbstractNode {
    private IObject obj;

    private IClass type;

    private boolean hasCheckedForChildren;

    public ReturnValueNode(IClass type, IObject obj) {
        this.type = type;
        this.obj = obj;
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getName()
     */
    public String getName() {
        return ObjBenchPlugin.getResourceString("ReturnValueNode.returnValue"); //$NON-NLS-1$
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getValue()
     */
    public IObject getValue() {
        return obj;
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getValueAsString()
     */
    public String getValueAsString() {
        if (type.isArray()) {
            IArray valArray = ((IArray) obj);
            try {
                return MessageFormat
                        .format(
                                ObjBenchPlugin
                                        .getResourceString("FieldNode.valueAsString"), new Object[] { //$NON-NLS-1$
                                type.getComponentType().getName(),
                                        Integer.toString(valArray.getLength()) });
            } catch (RemoteException re) {
                return MessageFormat
                        .format(
                                ObjBenchPlugin
                                        .getResourceString("FieldNode.errorValueAsString"), new Object[] { Integer //$NON-NLS-1$
                                        .toString(valArray.getLength()) });
            }
        } else if (type.isPrimitive())
            return obj.toString();
        else
            try {
                if (type.equals(ObjBenchPlugin.getDefault().getClassManager()
                        .getClass(String.class.getName())))
                    return '"' + obj.toString() + '"';
                try {
                    return obj.getIClass().getName();
                } catch (RemoteException re) {
                    ObjBenchLogger.logError(re);
                    return ObjBenchPlugin.getResourceString("FieldNode.type"); //$NON-NLS-1$
                }
            } catch (ClassNotFoundException cnfe) {
                // this should never happen, log as an error and return junk
                ObjBenchLogger.logError(cnfe);
                return ObjBenchPlugin.getResourceString("FieldNode.type"); //$NON-NLS-1$
            }

    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getType()
     */
    public IClass getType() {
        return type;
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getChildren()
     */
    public Node[] getChildren() {
        if (!hasCheckedForChildren) {
            hasCheckedForChildren = true;
            if (null != getValue()) {
                if (type.isArray()) {
                    IArray valArray = (IArray) obj;
                    for (int i = 0; i < valArray.getLength(); i++)
                        addChild(new ArrayItemNode(valArray, i, type));
                } else if (!type.isPrimitive()) {
                    IField[] fields = null;
                    try {
                        fields = obj.getIClass().getDeclaredFields();
                    } catch (RemoteException re) {
                        DetailsDialogUtil.openError(Display.getCurrent()
                                .getActiveShell(), re);
                        ObjBenchLogger.logError(re.getMessage(), re);
                    }
                    for (int i = 0; i < fields.length; i++)
                        addChild(new FieldNode(obj, fields[i]));
                }
            }
        }
        return super.getChildren();
    }

    /**
     * @return <code>true</code> if the object is non-null and any of the
     *         following are true:
     *         <ol>
     *         <li>the object is an array with a length greater than 0
     *         <li>the object is not-primitive and has at least one declared
     *         field
     *         </ol>
     * @see Node#hasChildren()
     */
    public boolean hasChildren() {
        if (null == getValue())
            return false;
        else if (type.isArray() && ((IArray) obj).getLength() > 0)
            return true;
        else
            try {
                return (!type.isPrimitive() && obj.getIClass()
                        .getDeclaredFields().length > 0);
            } catch (RemoteException re) {
                ObjBenchLogger.logError(re);
                return false;
            }
    }
    
    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getImage()
     */
    public Image getImage() {
        return JavaUI.getSharedImages().getImage(
                ISharedImages.IMG_OBJS_LOCAL_VARIABLE);
    }

}