/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 29, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 	  Justin Braga - Implemented getConstructors
 *    Justin Braga - getConstructors now throws CMException
 * 	  Justin Braga - Fixed getSuperclass to return null when no super
 *					 Implemented getMethods()
 *					 Moved in toString()
 *    Liam Morley - changed getSuperclass back, now all null classes return null
 *					getIClass() now returns the current instance
 * $Id: JEMClass.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.jdt.core.Signature;
import org.eclipse.jem.internal.proxy.core.IArrayBeanTypeProxy;
import org.eclipse.jem.internal.proxy.core.IBeanTypeProxy;
import org.eclipse.jem.internal.proxy.core.IConstructorProxy;
import org.eclipse.jem.internal.proxy.core.IFieldProxy;
import org.eclipse.jem.internal.proxy.core.IMethodProxy;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;
import org.eclipse.jface.preference.IPreferenceStore;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IConstructor;
import edu.wpi.ebob.model.cm.IField;
import edu.wpi.ebob.model.cm.IMethod;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.preferences.OBPreferencePage;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class JEMClass extends JEMMember implements IClass {

    private static Map classes = new WeakHashMap();

    /**
     * @see edu.wpi.ebob.model.cm.IObject#getIClass()
     */
    static JEMClass getClass(IBeanTypeProxy jemClass) {
        if (null == jemClass)
            return null;
        if (classes.containsKey(jemClass))
            return (JEMClass) classes.get(jemClass);
        JEMClass clazz = new JEMClass(jemClass);
        classes.put(jemClass, clazz);
        return clazz;
    }

    private String cachedClassName;

    private boolean cachedClassNamePref;

    private IBeanTypeProxy jemClass;

    private JEMClass(IBeanTypeProxy jemClass) {
        super(jemClass);
        this.jemClass = jemClass;
    }

    private IBeanTypeProxy[] getBeanTypeProxies(IClass[] iclasses) {
        IBeanTypeProxy[] beanTypeProxies = new IBeanTypeProxy[iclasses.length];
        for (int i = 0; i < iclasses.length; i++) {
            JEMClass clazz = (JEMClass) iclasses[i];
            beanTypeProxies[i] = clazz.jemClass;
        }
        return beanTypeProxies;
    }

    IBeanTypeProxy getBeanTypeProxy() {
        return jemClass;
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#getComponentType()
     */
    public IClass getComponentType() {
        if (jemClass instanceof IArrayBeanTypeProxy) {
            IArrayBeanTypeProxy classArray = (IArrayBeanTypeProxy) jemClass;
            return JEMClass.getClass(classArray.getComponentType());
        } else
            return this;
    }

    /**
     * @see IClass#getConstructor(IClass[])
     */
    public IConstructor getConstructor(IClass[] parameterTypes) {
        String[] paramTypes = new String[parameterTypes.length];
        for (int i = 0; i < paramTypes.length; i++) {
            paramTypes[i] = ((JEMClass) parameterTypes[i]).jemClass
                    .getFormalTypeName();
        }
        return JEMConstructor.getConstructor(jemClass
                .getConstructorProxy(paramTypes), jemClass);
    }

    /**
     * @throws RemoteException
     * @see edu.wpi.ebob.model.cm.IClass#getConstructors()
     */
    public IConstructor[] getConstructors() throws RemoteException {
        IConstructorProxy[] constructors = jemClass.getConstructors();

        IConstructor[] cons = new IConstructor[constructors.length];
        for (int i = 0; i < cons.length; i++) 
            cons[i] = JEMConstructor.getConstructor(constructors[i], jemClass);
        return cons;
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#getDeclaredField(java.lang.String)
     */
    public IField getDeclaredField(String name) {
        return JEMField.getField(jemClass.getDeclaredFieldProxy(name));
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#getDeclaredFields()
     */
    public IField[] getDeclaredFields() throws RemoteException {
        IFieldProxy[] declaredFields = jemClass.getDeclaredFields();
        IField[] returnFields = new IField[declaredFields.length];
        for (int i = 0; i < returnFields.length; i++)
            returnFields[i] = JEMField.getField(declaredFields[i]);
        return returnFields;
    }

    /**
     * currently does the same thing as getMethod. There is no underlying
     * "getDeclaredMethod" in JEM.
     * 
     * @see IClass#getDeclaredMethod(String, IClass[])
     */
    public IMethod getDeclaredMethod(String name, IClass[] parameterTypes) {
        return JEMMethod.getMethod(jemClass.getMethodProxy(name,
                getBeanTypeProxies(parameterTypes)));
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#getField(java.lang.String)
     */
    public IField getField(String name) {
        return JEMField.getField(jemClass.getFieldProxy(name));
    }

    /**
     * Returns <code>this</code>.
     * 
     * @see edu.wpi.ebob.model.cm.IObject#getIClass()
     */
    public IClass getIClass() {
        return this;
    }

    /**
     * @throws RemoteException
     * @see IClass#getMethod(String, IClass[])
     */
    public IMethod getMethod(String name, IClass[] parameterTypes)
            throws RemoteException {
        return JEMMethod.getMethod(jemClass.getMethodProxy(name,
                getBeanTypeProxies(parameterTypes)));
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#getMethods()
     */
    public IMethod[] getMethods() throws RemoteException {
        IMethodProxy[] methods = jemClass.getMethods();
        IMethod[] returnMethods = new IMethod[methods.length];
        for (int i = 0; i < returnMethods.length; i++)
            returnMethods[i] = JEMMethod.getMethod(methods[i]);
        return returnMethods;
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#getSuperclass()
     */
    public IClass getSuperclass() {
        return getClass(jemClass.getSuperBeanTypeProxy());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#isArray()
     */
    public boolean isArray() {
        return jemClass.isArray();
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#isPrimitive()
     */
    public boolean isPrimitive() {
        return jemClass.isPrimitive();
    }

    /**
     * @see edu.wpi.ebob.model.cm.IClass#newInstance()
     */
    public IObject newInstance() throws RemoteException {
        try {
            return JEMObject.getObject(jemClass.newInstance());
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        IPreferenceStore store = ObjBenchPlugin.getDefault()
                .getPreferenceStore();
        boolean useFQName = store.getBoolean(OBPreferencePage.P_FQ_CLASS_NAMES);

        if (cachedClassName != null && useFQName == cachedClassNamePref)
            return cachedClassName;

        String fqName = jemClass.getFormalTypeName();

        if (useFQName)
            return fqName;
        else
            return Signature.getSimpleName(fqName);
    }
}