/*******************************************************************************
 * Copyright (c) 2004, 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 29, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *	  Justin Braga - getClass doesn't throw CMException 
 *
 * $Id: JEMClassManager.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.text.MessageFormat;
import java.util.ArrayList;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jem.internal.proxy.core.IArrayBeanProxy;
import org.eclipse.jem.internal.proxy.core.IBeanTypeProxy;
import org.eclipse.jem.internal.proxy.core.IStandardBeanTypeProxyFactory;
import org.eclipse.jem.internal.proxy.core.ProxyFactoryRegistry;
import org.eclipse.jem.internal.proxy.core.ProxyLaunchSupport;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;
import org.eclipse.jface.util.ListenerList;
import org.eclipse.swt.widgets.Display;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.jem.operations.JEMAdditionOperation;
import edu.wpi.ebob.internal.jem.operations.JEMAlgebraicNegationOperation;
import edu.wpi.ebob.internal.jem.operations.JEMBooleanNegationOperation;
import edu.wpi.ebob.internal.jem.operations.JEMDivisionOperation;
import edu.wpi.ebob.internal.jem.operations.JEMModulusOperation;
import edu.wpi.ebob.internal.jem.operations.JEMMultiplicationOperation;
import edu.wpi.ebob.internal.jem.operations.JEMSubtractionOperation;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.ClassManagerEvent;
import edu.wpi.ebob.model.cm.IArray;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IClassManager;
import edu.wpi.ebob.model.cm.IClassManagerListener;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.IOperation;

/**
 * Uses the Java EMF Model for performing remote operations.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public class JEMClassManager implements IClassManager {
    // addition operation
    private static final IOperation ADD_OP = new JEMAdditionOperation();

    // subtraction operation
    private static final IOperation SUBTRACT_OP = new JEMSubtractionOperation();

    // multiplication operation
    private static final IOperation MULT_OP = new JEMMultiplicationOperation();

    // division operation
    private static final IOperation DIV_OP = new JEMDivisionOperation();

    // modulus operation
    private static final IOperation MOD_OP = new JEMModulusOperation();

    // algebraic negation operation
    private static final IOperation ALG_NEG_OP = new JEMAlgebraicNegationOperation();
    
    // boolean negation operation
    private static final IOperation BOOL_NEG_OP = new JEMBooleanNegationOperation();

    // the registry
    private ProxyFactoryRegistry registry;

    // the associated project
    private IJavaProject project;

    private boolean isDirty;

    private ListenerList listeners = new ListenerList(3);

    private JavaClassChangeListener javaClassChangeListener;

    /**
     * Tracks when changes are made to java classes in the current project.
     * 
     * @author Liam Morley
     *  
     */
    private class JavaClassChangeListener implements IResourceChangeListener {
        public void resourceChanged(IResourceChangeEvent event) {
            // if the class manager isn't associated, don't worry about it
            if (null == project)
                return;

            // if this is PRE_DELETE for the project, we need to
            // disassociate first
            if (event.getType() == IResourceChangeEvent.PRE_DELETE) {
                if (event.getResource() == project.getProject()) {
                    // make sure this is run before project is deleted
                    Display.getDefault().syncExec(new Runnable() {
                        public void run() {
                            ObjBenchPlugin objBenchPlugin = ObjBenchPlugin
                                    .getDefault();
                            ObjectBench bench = objBenchPlugin.getBench();
                            IClassManager cm = objBenchPlugin.getClassManager();

                            bench.removeAll();
                            cm.disassociateProject();
                        }
                    });

                    return;
                }
            }
            // now only interested in post-change
            else if (event.getType() != IResourceChangeEvent.POST_CHANGE)
                return;

            // tests whether the entire project has been deleted

            // is the changed resource underneath a source directory in the
            // current project?
            IResourceDelta srcDelta = null;
            try {
                IResourceDelta rootDelta = event.getDelta();

                boolean isSource = false;
                IPackageFragmentRoot[] packageFragmentRoots = project
                        .getPackageFragmentRoots();
                for (int i = 0; i < packageFragmentRoots.length; i++) {
                    IPackageFragmentRoot root = packageFragmentRoots[i];
                    // is this fragment root a source fragment, and is it
                    // part
                    // of this change?
                    if (root.getKind() == IPackageFragmentRoot.K_SOURCE) {
                        srcDelta = rootDelta.findMember(root.getPath());
                        if (null != srcDelta) {
                            // a source element has been changed
                            isSource = true;
                            break;
                        }
                    }
                }
                // no source element has been changed, return
                if (!isSource)
                    return;
            } catch (JavaModelException jme) {
                // we can't get a handle to the pkg fragment, log and return
                ObjBenchLogger.logWarning(jme);
                return;
            }

            final ArrayList changed = new ArrayList();
            IResourceDeltaVisitor visitor = new IResourceDeltaVisitor() {
                public boolean visit(IResourceDelta delta) {
                    // only interested in changed/removed resources (not
                    // added)
                    if (delta.getKind() != IResourceDelta.CHANGED
                            && delta.getKind() != IResourceDelta.REMOVED)
                        return true;
                    // only interested in content changes
                    if ((delta.getFlags() & IResourceDelta.CONTENT) == 0)
                        return true;

                    IResource resource = delta.getResource();
                    // only interested in java files
                    if (resource.getType() == IResource.FILE
                            && "java".equalsIgnoreCase(resource //$NON-NLS-1$
                                    .getFileExtension())) {
                        changed.add(resource);
                    }
                    return true;
                }
            };
            try {
                srcDelta.accept(visitor);
            } catch (CoreException ce) {
                ObjBenchLogger.logError(ce);
            }

            if (changed.size() != 0)
                setDirty(true);
        }
    }

    public JEMClassManager() {
        javaClassChangeListener = new JavaClassChangeListener();
        ResourcesPlugin.getWorkspace().addResourceChangeListener(
                javaClassChangeListener,
                IResourceChangeEvent.PRE_DELETE
                        | IResourceChangeEvent.POST_CHANGE);
    }

    /**
     * @see IClassManager#addClassManagerListener(IClassManagerListener)
     */
    public void addClassManagerListener(IClassManagerListener listener) {
        listeners.add(listener);
    }

    /**
     * @see IClassManager#removeClassManagerListener(IClassManagerListener)
     */
    public void removeClassManagerListener(IClassManagerListener listener) {
        listeners.remove(listener);
    }

    private void fireDirtyFlagEvent(ClassManagerEvent event) {
        Object[] listenerArray = listeners.getListeners();
        for (int i = 0; i < listenerArray.length; ++i)
            ((IClassManagerListener) listenerArray[i]).dirtyFlagSet(event);
    }

    /**
     * Associates this manager with a given Java project. Does nothing if the
     * current project is equal to the given project.
     * 
     * @see IClassManager#associateProject(IJavaProject)
     */
    public void associateProject(IJavaProject javaProject) {
        if (null == javaProject)
            throw new IllegalArgumentException();
        // don't reassociate if not necessary
        IProject rootProject = javaProject.getProject();
        if (null != project && project.getProject() == rootProject)
            return;

        if (null != registry)
            registry.terminateRegistry();
        this.project = javaProject;
        try {
            registry = ProxyLaunchSupport
                    .startImplementation(
                            rootProject,
                            MessageFormat
                                    .format(
                                            ObjBenchPlugin
                                                    .getResourceString("JEMClassManager.objBenchTitle"), //$NON-NLS-1$
                                            new Object[] { rootProject
                                                    .getName() }), null, null);
        } catch (CoreException ce) {
            // This should only happen if javaProject is null; it's not.
            ObjBenchLogger.logError(ce);
        }
    }

    /**
     * Returns whether this manager is currently associated with a valid
     * project.
     * 
     * @see IClassManager#isValid()
     */
    public boolean isValid() {
        return (null != registry && registry.isValid());
    }

    /**
     * @see IClassManager#isDirty()
     */
    public boolean isDirty() {
        return isDirty;
    }

    /**
     * Sets the dirty flag.
     */
    private void setDirty(boolean isDirty) {
        fireDirtyFlagEvent(new ClassManagerEvent(project, this.isDirty, isDirty));
        this.isDirty = isDirty;
    }

    /**
     * Cancels the association with the given project. Does nothing if a project
     * is not currently associated.
     * 
     * @see IClassManager#disassociateProject()
     */
    public void disassociateProject() {
        if (isValid()) {
            registry.terminateRegistry();
            setDirty(false);
            project = null;
        } else {
            //            (new RuntimeException()).printStackTrace();
            ObjBenchLogger
                    .logWarning(ObjBenchPlugin
                            .getResourceString("JEMClassManager.disassociatedBeforeAssociated")); //$NON-NLS-1$
        }
    }

    /**
     * Retrieves a class with the given name.
     * 
     * @see IClassManager#getClass(String)
     */
    public IClass getClass(String name) throws ClassNotFoundException {
        if (!isValid())
            throw new IllegalStateException();
        IStandardBeanTypeProxyFactory beanTypeProxyFactory = registry
                .getBeanTypeProxyFactory();
        IBeanTypeProxy type = beanTypeProxyFactory.getBeanTypeProxy(name);
        // classes in java.lang don't need to be fully qualified
        if (null == type && name.indexOf('.') == -1)
            type = beanTypeProxyFactory.getBeanTypeProxy("java.lang." + name); //$NON-NLS-1$
        // if we still can't find the type, throw a CNFE
        if (null == type)
            throw new ClassNotFoundException(name);
        return JEMClass.getClass(type);
    }

    /**
     * Returns an object of type <code>returnType</code> based on the given
     * <code>expression</code>, if such an expression can resolve to that
     * returnType.
     * 
     * @throws RemoteException
     * 
     * @see IClassManager#resolveObject(IClass, String)
     */
    public IObject resolveObject(IClass returnType, String expression)
            throws ClassCastException, InstantiationException, RemoteException {
        if (!isValid())
            throw new IllegalStateException();
        JEMClass jemClass = (JEMClass) returnType;
        try {
            return JEMObject.getObject(jemClass.getBeanTypeProxy().newInstance(
                    expression));
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see IClassManager#resolveArray(IClass, IObject[])
     */
    public IArray resolveArray(IClass type, IObject[] elements)
            throws RemoteException {
        if (!isValid())
            throw new IllegalStateException();
        // get length of array
        int len = elements.length;
        // get type of array
        IBeanTypeProxy typeProxy = ((JEMClass) type).getBeanTypeProxy();
        try {
            // initialize array
            IArrayBeanProxy arrayProxy = registry.getBeanProxyFactory()
                    .createBeanProxyWith(typeProxy, len);
            // set array values
            for (int i = 0; i < len; i++)
                arrayProxy.set(((JEMObject) elements[i]).jemObject, i);
            return new JEMArray(arrayProxy);
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see IClassManager#resolveValue(boolean)
     */
    public IObject resolveValue(boolean value) {
        if (!isValid())
            throw new IllegalStateException();
        return JEMObject.getObject(registry.getBeanProxyFactory()
                .createBeanProxyWith(value));
    }

    /**
     * @see IClassManager#resolveValue(int)
     */
    public IObject resolveValue(int value) {
        return JEMObject.getObject(registry.getBeanProxyFactory()
                .createBeanProxyWith(value));
    }

    /**
     * @see IClassManager#resolveValue(double)
     */
    public IObject resolveValue(double value) {
        return JEMObject.getObject(registry.getBeanProxyFactory()
                .createBeanProxyWith(value));
    }

    /**
     * @see IClassManager#resolveValue(char)
     */
    public IObject resolveValue(char value) {
        return JEMObject.getObject(registry.getBeanProxyFactory()
                .createBeanProxyWith(value));
    }

    /**
     * @see IClassManager#resolveValue(String)
     */
    public IObject resolveValue(String value) {
        return JEMObject.getObject(registry.getBeanProxyFactory()
                .createBeanProxyWith(value));
    }

    /**
     * Retrieves the project associated with this manager.
     * 
     * @see IClassManager#getAssociatedProject()
     */
    public IJavaProject getAssociatedProject() {
        return project;
    }

    /**
     * Returns the underlying JEM ProxyFactoryRegistry associated with this
     * manager.
     * 
     * @return Returns the registry.
     */
    public ProxyFactoryRegistry getRegistry() {
        return registry;
    }

    /**
     * performs the given operation with the given arguments.
     * 
     * @throws RemoteException
     * @see IClassManager#performOperation(IOperation, IObject[])
     */
    public IObject performOperation(IOperation op, IObject[] args)
            throws RemoteException {
        return op.performOperation(this, args);
    }

    /**
     * Performs <code>left + right</code>.
     * 
     * @see IClassManager#performAdditionOperation(IObject, IObject)
     */
    public IObject performAdditionOperation(IObject left, IObject right)
            throws RemoteException {
        return performOperation(ADD_OP, new IObject[] { left, right });
    }

    /**
     * Performs <code>left - right</code>.
     * 
     * @see IClassManager#performSubtractionOperation(IObject, IObject)
     */
    public IObject performSubtractionOperation(IObject left, IObject right)
            throws RemoteException {
        return performOperation(SUBTRACT_OP, new IObject[] { left, right });
    }

    /**
     * Performs <code>left * right</code>.
     * 
     * @see IClassManager#performMultiplicationOperation(IObject, IObject)
     */
    public IObject performMultiplicationOperation(IObject left, IObject right)
            throws RemoteException {
        return performOperation(MULT_OP, new IObject[] { left, right });
    }

    /**
     * Performs <code>left / right</code>.
     * 
     * @see IClassManager#performDivisionOperation(IObject, IObject)
     */
    public IObject performDivisionOperation(IObject left, IObject right)
            throws RemoteException {
        return performOperation(DIV_OP, new IObject[] { left, right });
    }

    /**
     * Performs <code>left % right</code>.
     * 
     * @see IClassManager#performModulusOperation(IObject, IObject)
     */
    public IObject performModulusOperation(IObject left, IObject right)
            throws RemoteException {
        return performOperation(MOD_OP, new IObject[] { left, right });
    }

    /**
     * @see IClassManager#performAlgebraicNegationOperation(IObject)
     */
    public IObject performAlgebraicNegationOperation(IObject operand)
            throws RemoteException {
        return performOperation(ALG_NEG_OP, new IObject[] { operand });
    }

    /**
     * @see IClassManager#performBooleanNegationOperation(IObject)
     */
    public IObject performBooleanNegationOperation(IObject operand)
            throws RemoteException {
        return performOperation(BOOL_NEG_OP, new IObject[] { operand });
    }
    /**
     * @see IClassManager#dispose()
     */
    public void dispose() {
        if (isValid())
            disassociateProject();
        ResourcesPlugin.getWorkspace().removeResourceChangeListener(
                javaClassChangeListener);
        listeners.clear();
    }
}