package edu.wpi.ebob.model;

import java.util.*;

/**
 * MethodInvokeInfo contains information about the methods and object involved in
 * a method invocation.  Maintains information regarding the fully qualified name 
 * of the calling object, the physical instance name of the calling object, 
 * the actuall name of the method within that object that was called, and a list
 * of parameters to the method.
 * 
 * @author bigben
 * @date 11/30/2005
 */
public class MethodInvokeInfo {	
	
	private String methodReturnType;
	private String fullyQualifiedMethodName;
	private String fullyQualifiedObjectName;
	private String instanceName;	
	private Vector paramList;	
		
	/**
	 * Constructs a MethodInvokeInfo object without any additonal information 
	 * regarding its parameters (an empty parameter list.)
	 *  
	 * @param fqmn The fully qualified name of the method that was called. Should look
	 * 			   something like "getInstanceOf"
	 * @param mrt  The return type of the method, like "void".
	 * @param fqon The fully qualified name of the calling object, ie "java.lang.String"
	 * @param in   The instance name of the object whose method was called.
	 */
	public MethodInvokeInfo(String fqmn, String mrt, String fqon, String in) {			
		fullyQualifiedMethodName = new String(fqmn);
		instanceName = new String (in);
		fullyQualifiedObjectName = new String(fqon);
		methodReturnType = new String(mrt);
		paramList = new Vector();
	}
	
	/**
	 * Constructs a MethodInvokeInfo object with information about its parameters.
	 * 
	 * @param fqmn   The fully qualified name of the method that was called.
	 * @param mrt    The return type of the method.
	 * @param fqon   The fully qualified name of the calling object.
	 * @param in     The instance name of the object whose method was called.
	 * @param params A vector of "MethodInvokeParamInfo", each of which represents an
	 * 				 individual parameter to the method invocation.
	 */
	public MethodInvokeInfo(String fqmn, String mrt, String fqon, String in, Vector params) {
		fullyQualifiedMethodName = new String(fqmn);
		instanceName = new String(in);
		methodReturnType = new String(mrt);
		fullyQualifiedObjectName = new String(fqon);
		paramList = new Vector();
		
		Iterator it = params.iterator(); 		
		// copy contents of param into local object
		while (it.hasNext()) {
			paramList.add((MethodInvokeParamInfo)it.next());
		}
	}

	/**
	 * Gets the fully qualified method name.
	 * @return the fully qualified method name.
	 */
	public String getFullyQualifiedMethodName() {
		return fullyQualifiedMethodName;
	}

	/**
	 * Sets the fully qualified method name.
	 * @param fullyQualifiedMethodName the fully qualified method name to set.
	 */
	public void setFullyQualifiedMethodName(String fullyQualifiedMethodName) {
		this.fullyQualifiedMethodName = fullyQualifiedMethodName;
	}

	/**
	 * Gets the fully qualified object name.
	 * @return the fully qualified method name.
	 */
	public String getFullyQualifiedObjectName() {
		return fullyQualifiedObjectName;
	}
	
	/**
	 * Sets the fully qualified object name.
	 * @param fqon the fully qualified object name to set.
	 */
	public void setFullyQualifiedObjectName(String fqon) {
		this.fullyQualifiedObjectName = fqon;
	}
	
	/**
	 * Gets the instance name of the calling object.
	 * @return the instance name of the calling object.
	 */
	public String getCallingObjectInstanceName() {
		return instanceName;
	}

	/**
	 * Sets the instance name of the calling object.
	 * @param instanceName the instanceName to set.
	 */
	public void setCallingObjectInstanceName(String instanceName) {
		this.instanceName = instanceName;
	}
	
	/**
	 * Adds a parameter to the internal representation of the method parameter list.
	 * @param param the parameter to add, of type MethodInvokeParamInfo.
	 */
	public void addParameter(MethodInvokeParamInfo param) {
		paramList.add(param);
	}

	/**
	 * Gets the return type of the method invocation.
	 * @return the return type of the method invocation.
	 */
	public String getMethodReturnType() {
		return methodReturnType;
	}

	/**
	 * Sets the return type of the method invocation.
	 * @param methodReturnType the return type to set.
	 */
	public void setMethodReturnType(String methodReturnType) {
		this.methodReturnType = methodReturnType;		
	}
	
	/**
	 * Returns an iterator into the parameter list. 
	 * @return an iterator
	 */
	public Iterator getParameterIterator() {
		return paramList.iterator();
	}
	
	
}
