/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 24, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: ObjectBench.java,v 1.2 2006/03/02 11:51:25 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.model;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import java.util.regex.Pattern;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.ListenerList;
import org.eclipse.swt.widgets.Display;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.cm.*;

import edu.wpi.ebob.internal.jem.*;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class ObjectBench {
    private class Obj {
        private String name;

        private IObject obj;

        /** objects that are dependent on this object. */
        private Set dependents;

        /**
         * objects that list this object as a dependent. These references are
         * saved so that this object can be easily removed as a dependent when
         * it is removed from the bench.
         */
        private Set dependers;

        public Obj(String name, IObject obj) {
            this.name = name;
            this.obj = obj;
            dependents = new HashSet();
            dependers = new HashSet();
        }

        /**
         * @return Returns the name of this object.
         */
        public String getName() {
            return name;
        }

        /**
         * @return Returns the object.
         */
        public IObject getObj() {
            return obj;
        }

        /**
         * Returns the dependents in a modifiable set.
         * 
         * @return the objects (of type Obj) that are dependent on this object.
         */
        public Set getDependents() {
            return dependents;
        }

        /**
         * @param dependent
         */
        public void addDependent(Obj dependent) {
            dependents.add(dependent);
            dependent.dependers.add(this);
        }

        /**
         * @param dependent
         */
        public void removeDependent(Obj dependent) {
            dependents.remove(dependent);
            dependent.dependers.remove(this);
        }

        public void dispose() {
            if (!dependers.isEmpty()) {
                // copy dependers into new set to avoid concurrent modification
                Set _dependers = new HashSet(dependers);
                for (Iterator iter = _dependers.iterator(); iter.hasNext();) {
                    Obj depender = (Obj) iter.next();
                    depender.removeDependent(this);
                }
                assert dependers.isEmpty();
                _dependers.clear();
            }
            dependents.clear();
        }

        /**
         * @return <code>true</code> if there is another object dependent on
         *         this object, <code>false</code> otherwise.
         */
        public boolean hasDependents() {
            return !dependents.isEmpty();
        }
    }

    private ListenerList listeners = new ListenerList(3);

    /** contains the actual mappings for name/object pairs. */
    private HashMap objs = new HashMap();

    /**
     * adds object <code>obj</code> with the given name <code>name</code>.
     * 
     * @param name
     *            the name of the object within the Object Bench.
     * @param obj
     *            the actual object in the bench.
     * @throws ObjectAdditionNotSupportedException
     *             if there is an object already with the given name, or if
     *             there was an exception thrown when adding the object to the
     *             view.
     */
    public void addObject(String name, IObject obj)
            throws ObjectAdditionNotSupportedException {
        if (objs.containsKey(name))
            throw new ObjectAdditionNotSupportedException(name);
        if (null == name || null == obj)
            throw new ObjectAdditionNotSupportedException(name,
                    new IllegalArgumentException(ObjBenchPlugin
                            .getResourceString("ObjectBench.null.name"))); //$NON-NLS-1$
        objs.put(name, new Obj(name, obj));
        fireObjectAddedEvent(new ObjectBenchEvent(name, obj));
    }

    /**
     * Adds the listener to receive events and immediately sends an event for
     * each currently existing object in the bench.
     * 
     * @param listener
     *            the listener
     * @see ObjectBenchListener
     */
    public void addObjectBenchListener(ObjectBenchListener listener) {
        addObjectBenchListener(listener, true);
    }

    /**
     * Adds the listener to receive events. If <code>sendCurrentObjects</code>
     * is <code>true</code>, the listener is immediately send an event for
     * every object currently existing in the Object Bench.
     * 
     * @param listener
     * @param sendCurrentObjects
     */
    public void addObjectBenchListener(ObjectBenchListener listener,
            boolean sendCurrentObjects) {
        listeners.add(listener);
        if (sendCurrentObjects)
            for (Iterator iter = objs.keySet().iterator(); iter.hasNext();) {
                String name = (String) iter.next();
                Obj obj = (Obj) objs.get(name);
                fireObjectAddedEvent(new ObjectBenchEvent(name, obj.getObj()));
            }
    }

    /**
     * Creates a dependency between two objects in the object bench.
     * 
     * @param name
     *            the name of the independent object.
     * @param dependent
     *            the name of the object that is dependent on <code>name</code>.
     */
    public void addDependency(String name, String dependent) {
        Obj independentObj = (Obj) objs.get(name);
        if (null == independentObj)
            throw new IllegalArgumentException(MessageFormat.format(
                    ObjBenchPlugin
                            .getResourceString("ObjectBench.unknownObject"), //$NON-NLS-1$
                    new Object[] { name }));
        Obj dependentObj = (Obj) objs.get(dependent);
        if (null == dependentObj)
            throw new IllegalArgumentException(MessageFormat.format(
                    ObjBenchPlugin
                            .getResourceString("ObjectBench.unknownObject"),//$NON-NLS-1$
                    new Object[] { dependent }));
        independentObj.addDependent(dependentObj);
    }

    /**
     * Retrieves the objects that are dependent on this <code>name</code>.
     * 
     * @param name
     *            the name of the independent object.
     * @return an array of the names of the dependent objects; of length 0 if
     *         there are no dependent objects.
     */
    public String[] getDependents(String name) {
        Obj o = (Obj) objs.get(name);
        if (null == o)
            throw new IllegalArgumentException(MessageFormat.format(
                    ObjBenchPlugin
                            .getResourceString("ObjectBench.unknownObject"), //$NON-NLS-1$
                    new Object[] { name }));
        // copy dependents to array
        Set dependentsSet = o.getDependents();
        String[] dependents = new String[dependentsSet.size()];
        int i = 0;
        for (Iterator iter = dependentsSet.iterator(); iter.hasNext();)
            dependents[i++] = ((Obj) iter.next()).getName();
        return dependents;
    }

    /**
     * @return <code>true</code> if there exists at least one object that is
     *         dependent on an object named <code>name</code>,
     *         <code>false</code> otherwise.
     */
    public boolean hasDependents(String name) {
        Obj o = (Obj) objs.get(name);
        if (null == o)
            throw new IllegalArgumentException(MessageFormat.format(
                    ObjBenchPlugin
                            .getResourceString("ObjectBench.unknownObject"), //$NON-NLS-1$
                    new Object[] { name }));
        return o.hasDependents();
    }

    /**
     * Returns <code>true</code> if the object with the given name exists in
     * the bench, <code>false</code> otherwise.
     * 
     * @param name
     *            the name of the object to check for.
     */
    public boolean containsObject(String name) {
        return objs.containsKey(name);
    }

    /**
     * alerts all listeners that an object has been added.
     * 
     * @param event
     *            the event that listeners should be alerted to.
     */
    private void fireObjectAddedEvent(ObjectBenchEvent event) {
    	//MessageDialog.openInformation(Display.getCurrent().getActiveShell(),
        //        "E-JUTR DEBUG DIALOG", "fireObjectAddedEvent"); 
    	Object[] listenerArray = listeners.getListeners();
        for (int i = 0; i < listenerArray.length; ++i)
            ((ObjectBenchListener) listenerArray[i]).objectAdded(event);        
    }

    /**
     * alerts all listeners that an object has been removed.
     * 
     * @param event
     *            the event that listeners should be alerted to.
     */
    private void fireObjectRemovedEvent(ObjectBenchEvent event) {
    	//MessageDialog.openInformation(Display.getCurrent().getActiveShell(),
        //        "E-JUTR DEBUG DIALOG", "fireObjectRemovedEvent");
        Object[] listenerArray = listeners.getListeners();
        for (int i = 0; i < listenerArray.length; ++i)
            ((ObjectBenchListener) listenerArray[i]).objectRemoved(event);
    }

    //TODO: 11/10/05 - BM
    /**
     * Alerts all listeners that a method has been called.  Translates from inner 
     * types found within edu.wpi.ebob.model.cm into the class MethodInvokeInfo to
     * avoid giving away too much information about the inner types within EBOB.
     * 
     * @param method		
     * 				the method being called
     * @param object
     * 				the object being acted on.
     * @param parameters
     * 				parameters to the method call.
     */
    public void fireMethodCalledEvent(IMethod method, IObject object, IObject [] parameters) {
    	//MessageDialog.openInformation(Display.getCurrent().getActiveShell(),
        //        "E-JUTR DEBUG DIALOG", "fireMethodCalledEvent"); 
    	    	
    	// now to translate from ebob's internal types into our interface classes 
    	// MethodInvokeInfo and MethodInvokeParamInfo
    	
    	Vector paramList = new Vector();
    	
    	MethodInvokeInfo mii = null;
    	MethodInvokeParamInfo mpi = null;
    	String instanceName = null;
    	String methodName = null;
    	String retType = null;
    	String callObjName = null;
        	
    	// pollice fix
    	    	
    	Iterator it = objs.keySet().iterator();
    	
    	while (it.hasNext()) {
    		String name = (String) it.next();
    		Obj obj = (Obj) objs.get(name);
    		IObject iobj = obj.getObj();    		
    		if (iobj.equals(object)) {
    			instanceName = name;
    		}
    	}
    	String [] temp;
    	
    	
    	try {
	    	methodName   = method.getName();
			retType      = method.getReturnType().getName();
			
			temp = object.objToString().split("@");    		
    		callObjName  = new String(temp[0]);
  
    		for (int i = 0; i < parameters.length; i++) { // for each parameter        	
    			if (!((JEMObject)parameters[i]).getBeanProxy().toString().contains("REMBeanProxy")) { // if it is not a REMBeanProxy
	    			// this triggers for simple data types        		        		        
	        		mpi = new MethodInvokeParamInfo("", parameters[i].toString());        		
	        		paramList.add(mpi);
        		} else {
        			// this triggers for complex data types (objects)
        			String [] temp2 = parameters[i].toString().split("@");
        			String instHack = "";
        			it = objs.keySet().iterator();    	
    				while (it.hasNext()) {
			    		String n2 = (String) it.next();
			    		Obj obj = (Obj) objs.get(n2);
			    		IObject iobj = obj.getObj();    		
			    		if (iobj.equals(parameters[i])) {
			    			instHack = n2;
			    	    }
			        }        		
        			
        			mpi = new MethodInvokeParamInfo(temp2[0], instHack);
        			paramList.add(mpi);
        		}        		
        	}    		
    	} catch (RemoteException re) {
    		ObjBenchLogger.logError(re);
    	}    	 
			    	
    	mii = new MethodInvokeInfo(methodName, retType, callObjName, instanceName, paramList);     			 				       			                     	
    	
    	Object[] listenerArray = listeners.getListeners();
    	for (int i = 0; i < listenerArray.length; ++i) 
    		((ObjectBenchListener) listenerArray[i]).methodCalled(mii);
    }    
    //TODO: 11/10/05 - end edit
    
    /** Returns an array of the names of the objects associated with this bench. */
    public String[] getNames() {
        Set keySet = objs.keySet();
        String[] namesArray = new String[keySet.size()];
        return (String[]) keySet.toArray(namesArray);
    }

    /**
     * returns a handle to the object in the bench specified by name
     * <code>name</code>.
     * 
     * @param name
     *            the name of the object to retrieve.
     */
    public IObject getObject(String name) {
        return ((Obj) objs.get(name)).getObj();
    }

    /**
     * removes an object with the specified name from the Object Bench.
     * 
     * @param name
     *            the name of the object to remove.
     * @return <code>true</code> if the object was able to be removed,
     *         <code>false</code> otherwise.
     * @throws ObjectRemovalNotSupportedException
     *             if there is an object dependency obstructing the removal of
     *             the object, or if there was any exception thrown while trying
     *             to remove the object to the view.
     */
    public boolean removeObject(String name)
            throws ObjectRemovalNotSupportedException {
        if (!objs.containsKey(name))
            return false;
        // check for dependencies
        Obj o = (Obj) objs.get(name);
        if (o.hasDependents())
            throw new ObjectRemovalNotSupportedException(name);
        fireObjectRemovedEvent(new ObjectBenchEvent(name, ((Obj) objs
                .remove(name)).getObj()));
        o.dispose();
        o = null;
        return true;
    }

    /**
     * Removes the object from the object bench. Any dependent objects will also
     * be removed.
     * 
     * @param name
     *            the name of the object to remove.
     * @return <code>true</code> if the object was able to be removed,
     *         <code>false</code> otherwise.
     */
    public boolean removeObjectCascade(String name) {
        if (!objs.containsKey(name))
            return false;
        // check for dependencies
        Obj o = (Obj) objs.get(name);
        if (o.hasDependents()) {
            // copy dependents into new set to avoid concurrent modification
            Set dependents = new HashSet(o.getDependents());
            for (Iterator iter = dependents.iterator(); iter.hasNext();) {
                Obj dependent = (Obj) iter.next();
                removeObjectCascade(dependent.getName());
                dependent = null;
            }
        }
        o.dispose();
        fireObjectRemovedEvent(new ObjectBenchEvent(name, ((Obj) objs
                .remove(name)).getObj()));
        o = null;
        return true;
    }

    /**
     * Removes the object <code>name</code> from the Object Bench. Dependents
     * are not removed.
     * 
     * @param name
     *            the name of the object to remove.
     * @return <code>true</code> if the object was able to be removed,
     *         <code>false</code> otherwise.
     */
    public boolean removeObjectForced(String name) {
        if (!objs.containsKey(name))
            return false;
        // check for dependencies
        Obj o = (Obj) objs.get(name);
        o.dispose();
        fireObjectRemovedEvent(new ObjectBenchEvent(name, ((Obj) objs
                .remove(name)).getObj()));
        o = null;
        return true;
    }

    /**
     * Removes the specified object bench listener so that it no longer receives
     * events from the Object Bench.
     * 
     * @param listener
     *            the listener
     */
    public void removeObjectBenchListener(ObjectBenchListener listener) {
        listeners.remove(listener);
    }

    /**
     * Removes all objects of the specified class.
     * 
     * @param c
     *            the class of objects which you want to remove.
     * @return <code>true</code> if <b>all </b> objects of this class were
     *         able to be removed, <code>false</code> otherwise.
     * @throws ObjectRemovalNotSupportedException
     *             if there is an object dependency obstructing the removal of
     *             an object, or if there was any exception thrown while trying
     *             to add an object to the view. If there was an exception
     *             thrown, it can be retrieved by {@link Throwable#getCause()},
     *             however it is not guaranteed that all objects specified could
     *             not be added for that reason.
     */
    public boolean removeObjects(IClass c)
            throws ObjectRemovalNotSupportedException {
        Set keys = objs.keySet();
        Collection removals = new ArrayList();

        for (Iterator iter = keys.iterator(); iter.hasNext();) {
            String name = iter.next().toString();
            Obj obj = (Obj) objs.get(name);
            if (obj.getObj().getIClass().equals(c))
                removals.add(name);
        }

        // collection of names that are unable to be removed
        Collection errors = new ArrayList();
        // possible cause of being unable to remove objects
        Throwable cause = null;

        boolean isSuccess = true;
        boolean hasError = false;

        for (Iterator iter = removals.iterator(); iter.hasNext();) {
            String name = (String) iter.next();
            try {
                isSuccess &= this.removeObject(name);
            } catch (ObjectRemovalNotSupportedException e) {
                // try to remove all that you can, and report the errors
                // afterwards
                hasError = true;
                if (null != e.getCause()) {
                    cause = e.getCause();
                }
                errors.add(name);
            }
        }
        if (hasError) {
            if (cause == null)
                throw new ObjectRemovalNotSupportedException(errors);
            else
                throw new ObjectRemovalNotSupportedException(errors, cause);
        }
        return isSuccess;
    }
    /**
     * Clears the Object Bench, removing all objects from the bench.
     */
    public void removeAll() {
        String[] names = getNames();
        for (int i = 0; i < names.length; i++) {
            removeObjectForced(names[i]);
        }
    }
}