/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 29, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *	  Justin Braga - getClass doesn't throw CMException
 * 
 * $Id: IClassManager.java,v 1.1 2006/01/23 16:07:05 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.model.cm;

import org.eclipse.jdt.core.IJavaProject;

/**
 * Performs remote operations on a JVM and retains references for obtaining
 * remote classes and objects.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public interface IClassManager {

    public void addClassManagerListener(IClassManagerListener listener);

    public void removeClassManagerListener(IClassManagerListener listener);

    /**
     * Associates this manager with a given java project. Does nothing if the
     * current project is equal to the given project.
     * 
     * @param javaProject
     *            the given project, can not be <code>null</code>.
     */
    public void associateProject(IJavaProject javaProject);

    /**
     * Retrieves the associated project, if one exists.
     * 
     * @return the project associated with this manager.
     */
    public IJavaProject getAssociatedProject();

    /**
     * States whether the manager is currently associated with a project.
     * 
     * @return <code>true</code> if the manager is currently associated with a
     *         project, <code>false</code> otherwise.
     */
    public boolean isValid();

    /**
     * States whether the underlying class structure may be out of sync with the
     * class manager.
     * 
     * @return <code>true</code> if the dirty flag has been set,
     *         <code>false</code> otherwise.
     */
    public boolean isDirty();

    /**
     * Sets the dirty flag for this class manager.
     * 
     * @param isDirty
     *            <code>true</code> if the underlying class structure may be
     *            out of sync with this class manager, <code>false</code>
     *            otherwise.
     */
    //    public void setDirty(boolean isDirty);
    /**
     * Removes any given association with any project. Does nothing if no such
     * association currently exists.
     */
    public void disassociateProject();

    /**
     * Retrieves a class with the given name.
     * 
     * @param name
     *            the fully-qualified class name. Can also be "int" or another
     *            primitive type.
     * @return the class if it exists.
     * @throws IllegalStateException
     *             if the manager is not currently associated with a project.
     */
    public IClass getClass(String name) throws ClassNotFoundException;

    /**
     * Returns an object of type <code>returnType</code> based on the given
     * <code>expression</code>, if such an expression can resolve to that
     * returnType.
     */
    public IObject resolveObject(IClass type, String expression)
            throws ClassCastException, InstantiationException, RemoteException;

    /**
     * Returns an array object using the IArray form.
     * 
     * @param type
     *            the type of the array
     * @param elements
     *            the elements of the array
     * @return the converted IArray object
     */
    public IArray resolveArray(IClass type, IObject[] elements)
            throws RemoteException;

    /**
     * Returns an IObject representing a primitive boolean value.
     * 
     * @param value
     *            the boolean value, <code>true</code> or <code>false</code>.
     * @return an IObject representing a primitive boolean value.
     */
    public IObject resolveValue(boolean value);

    /**
     * Returns an IObject representing a primitive integer value.
     * 
     * @param value
     *            the integer value.
     * @return an IObject representing a primitive integer value.
     */
    public IObject resolveValue(int value);

    /**
     * Returns an IObject representing a primitive double value.
     * 
     * @param value
     *            the double value.
     * @return an IObject representing a primitive double value.
     */
    public IObject resolveValue(double value);

    /**
     * Returns an IObject representing a primitive char value.
     * 
     * @param value
     *            the char value.
     * @return an IObject representing a primitive char value.
     */
    public IObject resolveValue(char value);

    /**
     * Returns an IObject representing a primitive String value.
     * 
     * @param value
     *            the String value.
     * @return an IObject representing a primitive String value.
     */
    public IObject resolveValue(String value);

    /**
     * performs the given operation with the given arguments.
     * 
     * @throws RemoteException
     *             if there is an error performing the operation.
     */
    public IObject performOperation(IOperation op, IObject args[])
            throws RemoteException;

    /**
     * Performs <code>left + right</code>.
     */
    public IObject performAdditionOperation(IObject left, IObject right)
            throws RemoteException;

    /**
     * Performs <code>left - right</code>.
     */
    public IObject performSubtractionOperation(IObject left, IObject right)
            throws RemoteException;

    /**
     * Performs <code>left * right</code>.
     */
    public IObject performMultiplicationOperation(IObject left, IObject right)
            throws RemoteException;

    /**
     * Performs <code>left / right</code>.
     */
    public IObject performDivisionOperation(IObject left, IObject right)
            throws RemoteException;

    /**
     * Performs <code>left % right</code>.
     */
    public IObject performModulusOperation(IObject left, IObject right)
            throws RemoteException;

    /**
     * Performs <code>-operand</code>.
     */
    public IObject performAlgebraicNegationOperation(IObject operand)
            throws RemoteException;

    /**
     * Performs <code>!operand</code>.
     */
    public IObject performBooleanNegationOperation(IObject operand)
            throws RemoteException;

    /** Clears up any resources held by the class manager. */
    public void dispose();
}