/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Feb 13, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *    Justin Braga - Catching IllegalStateException from getClass
 * 
 * $Id: Parser.java,v 1.1 2006/01/23 16:08:23 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.model.parser;

import java.text.MessageFormat;
import java.util.Collection;
import java.util.Iterator;
import java.util.Stack;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IClassManager;
import edu.wpi.ebob.model.cm.IConstructor;
import edu.wpi.ebob.model.cm.IField;
import edu.wpi.ebob.model.cm.IMethod;
import edu.wpi.ebob.model.cm.IObject;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class Parser {
    private Token ct; // current token

    private Iterator tokens;

    // (1+2) * 3 ====> 3 2 1 + *
    private Stack stack;

    private IClassManager cm;

    private ObjectBench bench;

    public Parser() {
        stack = new Stack();
        ObjBenchPlugin objBenchPlugin = ObjBenchPlugin.getDefault();
        cm = objBenchPlugin.getClassManager();
        bench = objBenchPlugin.getBench();
        if (!cm.isValid())
            throw new IllegalStateException(ObjBenchPlugin
                    .getResourceString("Parser.cmIsNotValid")); //$NON-NLS-1$
    }

    public IObject parse(Collection tokens) throws ParserException {
        this.tokens = tokens.iterator();
        ct = nextToken();
        while (ct.getType() != TokenType.EOI)
            additiveExpr();

        assert stack.size() <= 1;
        if (stack.isEmpty())
            return null;
        else
            return (IObject) stack.pop();
    }

    /**
     * AdditiveExpr = MultExpr (AddOp MultExpr)*
     * 
     * @throws ParserException
     */
    private void additiveExpr() throws ParserException {
        multiplicativeExpr();
        while (ct.getType() == TokenType.PLUS
                || ct.getType() == TokenType.MINUS) {

            // keep reference to which operation it is
            TokenType addOp = ct.getType();
            ct = nextToken();
            multiplicativeExpr();

            // perform operation
            performInfixOp(addOp);
        }
    }

    /**
     * MultExpr = UnaryExpr (MultOp UnaryExpr)*
     * 
     * @throws ParserException
     */
    private void multiplicativeExpr() throws ParserException {
        unaryExpr();
        while (ct.getType() == TokenType.STAR
                || ct.getType() == TokenType.SLASH
                || ct.getType() == TokenType.MOD) {

            // keep reference to which operation it is
            TokenType multOp = ct.getType();
            ct = nextToken();
            unaryExpr();

            // perform operation
            performInfixOp(multOp);
        }
    }

    /**
     * UnaryExpr = (BoolOp | AddOp)? PostfixExpr
     * 
     * @throws ParserException
     *  
     */
    private void unaryExpr() throws ParserException {
        TokenType addOp = null;
        if (ct.getType() == TokenType.PLUS || ct.getType() == TokenType.MINUS || ct.getType() == TokenType.EXCLAMATION) {
            addOp = ct.getType();
            ct = nextToken();
        }
        postfixExpr();
        // did we have leading addOp?
        if (addOp != null && addOp == TokenType.MINUS)
            performPrefixOp(addOp);
    }

    /**
     * PostfixExpr = PrimaryExpr (. Identifier ( LPAR ArgList RPAR)?)? | NewExpr
     * 
     * @throws ParserException
     *  
     */
    private void postfixExpr() throws ParserException {
        // NewExpr
        if (ct.getType() == TokenType.KEYWORD && ct.getValue().equals("new")) { //$NON-NLS-1$
            newExpr();
        }
        // PrimaryExpr (. Identifier ( LPAR ArgList RPAR)?)?
        else {
            primaryExpr();
            while (ct.getType() == TokenType.DOT) {
                ct = nextToken();
                assertCondition(TokenType.ID);

                // keep a reference to this token
                Token marker = ct;

                ct = nextToken();
                if (ct.getType() == TokenType.LPAR) {
                    // method
                    int stackSize = stack.size();
                    ct = nextToken();
                    if (ct.getType() != TokenType.RPAR) {
                        argList();
                        assertCondition(TokenType.RPAR);
                    }

                    // get arguments
                    assert stack.size() >= stackSize;
                    int size = stack.size() - stackSize;
                    IClass[] types = new IClass[size];
                    IObject[] args = new IObject[size];
                    for (int i = size - 1; i >= 0; i--) {
                        args[i] = (IObject) stack.pop();
                        types[i] = args[i].getIClass();
                    }

                    // resolve method
                    IObject obj = (IObject) stack.pop();
                    IClass objType = obj.getIClass();
                    IMethod method;
                    try {
                        method = objType.getMethod(marker.getValue(), types);
                    } catch (RemoteException re) {
                        throw new ParserException(re);
                    }

                    // make sure method is non-null
                    assertCondition(
                            null != method,
                            MessageFormat
                                    .format(
                                            ObjBenchPlugin
                                                    .getResourceString("Parser.unknownMethod"), new String[] { marker //$NON-NLS-1$
                                                    .getValue() }));

                    // invoke method and push result on the stack
                    try {
                        IObject result = method.invoke(obj, args);
                        stack.push(result);
                    } catch (RemoteException re) {
                        throw new ParserException(re);
                    }
                    ct = nextToken();
                } else {
                    // it's a field
                    IObject obj = (IObject) stack.pop();
                    IClass objType = obj.getIClass();

                    // validate field exists
                    IField field = objType.getField(marker.getValue());
                    assertCondition(null != field, ObjBenchPlugin
                            .getResourceString("Parser.unknownField")); //$NON-NLS-1$

                    try {
                        IObject result = field.get(obj);
                        stack.push(result);
                    } catch (RemoteException re) {
                        throw new ParserException(re);
                    }
                }
            }
        }
    }

    /**
     * PrimaryExpr = IdentPrimary | Constant | true | false | null | NewExpr |
     * LPAR AdditiveExpr RPAR
     * 
     * @throws ParserException
     */
    private void primaryExpr() throws ParserException {
        // LPAR AdditiveExpr RPAR
        if (ct.getType() == TokenType.LPAR) {
            ct = nextToken();
            if (ct.getType() != TokenType.RPAR) {
                additiveExpr();
                assertCondition(TokenType.RPAR);
            }
            ct = nextToken();
        }
        // NewExpr
        else if (ct.getType() == TokenType.KEYWORD
                && ct.getValue().equals("new")) //$NON-NLS-1$
            newExpr();
        // IdentPrimary | Constant | true | false | null
        else if (ct.getType() == TokenType.ID
                || ct.getType() == TokenType.STRING
                || ct.getType() == TokenType.CHAR
                || ct.getType() == TokenType.INT
                || ct.getType() == TokenType.FLOAT
                || (ct.getType() == TokenType.KEYWORD && (ct.getValue().equals(
                        "null") //$NON-NLS-1$
                        || ct.getValue().equals("true") || ct.getValue() //$NON-NLS-1$
                        .equals("false")))) { //$NON-NLS-1$
            stack.push(getObject(ct));
            ct = nextToken();
        } else
            throw new ParserException(MessageFormat.format(ObjBenchPlugin
                    .getResourceString("Parser.unexpectedToken"), //$NON-NLS-1$
                    new Object[] { ct.getType() }));
    }

    /**
     * NewExpr = new (Type LPAR ArgList RPAR | Type[] { ArgList })
     * 
     * @throws ParserException
     */
    private void newExpr() throws ParserException {
        ct = nextToken();
        assertCondition(ct.getType() == TokenType.ID
                || ct.getType() == TokenType.KEYWORD, ObjBenchPlugin
                .getResourceString("Parser.identifierExpected")); //$NON-NLS-1$

        StringBuffer className = new StringBuffer(ct.getValue());

        ct = nextToken();
        while (ct.getType() == TokenType.DOT) {
            className.append('.');
            ct = nextToken();
            assertCondition(TokenType.ID);
            className.append(ct.getValue());
            ct = nextToken();
        }

        IClass type;
        try {
            type = cm.getClass(className.toString());
        } catch (ClassNotFoundException cnfe) {
            throw new ParserException(cnfe);
        }

        boolean isArray = false;
        if (ct.getType() == TokenType.LBRACK) {
            ct = nextToken();
            assertCondition(TokenType.RBRACK);
            ct = nextToken();
            isArray = true;
        }

        // expected open and close symbols, depending on array status
        TokenType opener = (isArray) ? TokenType.LCURLY : TokenType.LPAR;
        TokenType closer = (isArray) ? TokenType.RCURLY : TokenType.RPAR;

        assertCondition(opener);

        int stackSize = stack.size();
        ct = nextToken();
        if (ct.getType() != closer) {
            argList();
            assertCondition(closer);
        }

        // get arguments
        assert stack.size() >= stackSize;
        int size = stack.size() - stackSize;
        IClass[] types = new IClass[size];
        IObject[] args = new IObject[size];
        for (int i = size - 1; i >= 0; i--) {
            args[i] = (IObject) stack.pop();
            types[i] = args[i].getIClass();
        }

        if (isArray) {
            try {
                stack.push(cm.resolveArray(type, args));
            } catch (RemoteException re) {
                throw new ParserException(re);
            }
        } else {
            // resolve constructor
            IConstructor constructor = type.getConstructor(types);
            try {
                stack.push(constructor.newInstance(args));
            } catch (RemoteException re) {
                throw new ParserException(re);
            }
        }
        ct = nextToken();
    }

    /**
     * ArgList = AdditiveExpr (, AdditiveExpr)*
     * 
     * @throws ParserException
     *  
     */
    private void argList() throws ParserException {
        additiveExpr();
        while (ct.getType() == TokenType.COMMA) {
            ct = nextToken();
            additiveExpr();
        }
    }

    private Token nextToken() {
        if (tokens.hasNext())
            return (Token) tokens.next();
        else
            return new Token(TokenType.EOI);
    }

    private void assertCondition(boolean condition, String msg)
            throws ParserException {
        if (!condition)
            throw new ParserException(msg);
    }

    private void assertCondition(TokenType expectedTokenType)
            throws ParserException {
        assertCondition(ct.getType() == expectedTokenType, MessageFormat
                .format(ObjBenchPlugin
                        .getResourceString("Parser.valueExpected"), //$NON-NLS-1$
                        new Object[] { expectedTokenType }));
    }

    private IObject getObject(Token token) throws ParserException {
        TokenType tt = token.getType();
        String value = token.getValue();
        try {
            if (tt == TokenType.INT)
                return cm.resolveValue(Integer.parseInt(value));
            else if (tt == TokenType.FLOAT)
                return cm.resolveValue(Double.parseDouble(value));
            else if (tt == TokenType.CHAR)
                return cm.resolveValue(value.toCharArray()[0]);
            else if (tt == TokenType.STRING)
                return cm.resolveValue(value);
            else if (tt == TokenType.KEYWORD) {
                if (value.equals("null")) //$NON-NLS-1$
                    return null;
                else if (value.equals("true") || value.equals("false")) //$NON-NLS-1$ //$NON-NLS-2$
                    return cm.resolveValue(Boolean.valueOf(value)
                            .booleanValue());
            } else if (tt == TokenType.ID) {
                // an ID is either an object on the bench or a static class
                // reference
                String[] names = bench.getNames();
                for (int i = 0; i < names.length; i++)
                    if (names[i].equals(value))
                        return bench.getObject(names[i]);
                IClass clazz = cm.getClass(value);
                if (null == clazz)
                    throw new ParserException(ObjBenchPlugin
                            .getResourceString("Parser.unknownIdentifier")); //$NON-NLS-1$
                return clazz;
            }
            System.err.println(tt);
            return null;
        } catch (Exception e) {
            throw new ParserException(e);
        }
    }

    private void performInfixOp(TokenType type) throws ParserException {
        if (stack.size() < 2)
            throw new ParserException(ObjBenchPlugin
                    .getResourceString("Parser.moreArgumentsNeeded")); //$NON-NLS-1$
        IObject right = (IObject) stack.pop();
        IObject left = (IObject) stack.pop();
        try {
            if (type == TokenType.PLUS)
                stack.push(cm.performAdditionOperation(left, right));
            else if (type == TokenType.MINUS)
                stack.push(cm.performSubtractionOperation(left, right));
            else if (type == TokenType.STAR)
                stack.push(cm.performMultiplicationOperation(left, right));
            else if (type == TokenType.SLASH)
                stack.push(cm.performDivisionOperation(left, right));
            else if (type == TokenType.MOD)
                stack.push(cm.performModulusOperation(left, right));
        } catch (Exception e) {
            throw new ParserException(e);
        }
    }

    private void performPrefixOp(TokenType type) throws ParserException {
        if (stack.size() < 1)
            throw new ParserException(ObjBenchPlugin
                    .getResourceString("Parser.moreArgumentsNeeded")); //$NON-NLS-1$
        IObject operand = (IObject) stack.pop();
        try {
            if (type == TokenType.MINUS)
                stack.push(cm.performAlgebraicNegationOperation(operand));
            else if (type == TokenType.EXCLAMATION)
                stack.push(cm.performBooleanNegationOperation(operand));
        } catch (Exception e) {
            throw new ParserException(e);
        }
    }
}