/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Feb 14, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: Scanner.java,v 1.1 2006/01/23 16:08:23 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.model.parser;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.jdt.core.JavaConventions;

import edu.wpi.ebob.ObjBenchPlugin;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class Scanner {
    private static final int IS_FLOAT = 1;

    private int state = 0;

    /**
     * scans the given source string and returns a list of tokens.
     * 
     * @param string
     *            the source string to parse; should be any valid Java
     *            expression.
     * @return a collection of {@link Token}s.
     * @throws IllegalArgumentException
     *             if <code>string</code> is equal to <code>null</code>.
     */
    public Collection scan(String string) {
        if (null == string)
            throw new IllegalArgumentException(ObjBenchPlugin
                    .getResourceString("Scanner.nullString")); //$NON-NLS-1$
        char[] chars = string.toCharArray();
        int len = chars.length;
        Collection tokens = new ArrayList();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < len; i++) {
            switch (chars[i]) {
            case '.':
                if (sb.length() > 0) {
                    // test to see if what we have so far is a number;
                    // if so, set state
                    if (isNumber(sb.toString())) {
                        state = IS_FLOAT;
                        break;
                    }
                    Token token = new Token(TokenType.ID);
                    token.setValue(sb.toString());
                    tokens.add(token);
                }
                tokens.add(new Token(TokenType.DOT));
                sb.setLength(0);
                break;
            case '(':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.LPAR));
                sb.setLength(0);
                break;
            case ')':
                if (sb.length() > 0)
                    tokens.addAll(scan(sb.toString()));
                tokens.add(new Token(TokenType.RPAR));
                sb.setLength(0);
                break;
            case '{':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.LCURLY));
                sb.setLength(0);
                break;
            case '}':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.RCURLY));
                sb.setLength(0);
                break;
            case '[':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.LBRACK));
                sb.setLength(0);
                break;
            case ']':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.RBRACK));
                sb.setLength(0);
                break;
            case ',':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.COMMA));
                sb.setLength(0);
                break;
            case '+':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.PLUS));
                sb.setLength(0);
                break;
            case '-':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.MINUS));
                sb.setLength(0);
                break;
            case '*':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.STAR));
                sb.setLength(0);
                break;
            case '%':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.MOD));
                sb.setLength(0);
                break;
            case '/':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.SLASH));
                sb.setLength(0);
                break;
            case '!':
                if (sb.length() > 0)
                    tokens.add(getToken(sb.toString()));
                tokens.add(new Token(TokenType.EXCLAMATION));
                sb.setLength(0);
                break;
            case '"':
                assert sb.length() == 0;
                while (++i < len && chars[i] != '"')
                    sb.append(chars[i]);
                if (i < len) { // we've found a matching ending '"'
                    Token token = new Token(TokenType.STRING);
                    token.setValue(sb.toString());
                    tokens.add(token);
                    sb.setLength(0);
                }
                break;
            case '\'':
                assert sb.length() == 0;
                while (i < len && chars[++i] != '\'')
                    sb.append(chars[i]);
                if (chars[i] == '\'') {
                    Token token = new Token(TokenType.CHAR);
                    token.setValue(sb.toString());
                    tokens.add(token);
                    sb.setLength(0);
                    break;
                } else
                    throw new RuntimeException(ObjBenchPlugin
                            .getResourceString("Scanner.illegalChar")); //$NON-NLS-1$
            case ' ':
                if (state == IS_FLOAT) {
                    Token token = new Token(TokenType.FLOAT);
                    token.setValue(sb.toString());
                    tokens.add(token);
                    state = 0;
                } else {
                    String val = sb.toString();
                    if (val.length() == 0)
                        break;
                    tokens.add(getToken(val));
                }
                sb.setLength(0);
                break;
            default:
                sb.append(chars[i]);
            }
        }
        if (sb.length() > 0) {
            if (state == IS_FLOAT) {
                Token token = new Token(TokenType.FLOAT);
                token.setValue(sb.toString());
                tokens.add(token);
            } else {
                String val = sb.toString();
                tokens.add(getToken(val));
            }
        }
        state = 0;
        return tokens;
    }

    private boolean isNumber(String str) {
        char[] chars = str.toCharArray();
        for (int i = 0; i < chars.length; i++)
            if (!Character.isDigit(chars[i]))
                return false;
        return true;
    }

    private Token getToken(String val) {
        Token token;
        if (Character.isDigit(val.charAt(0)))
            token = new Token(TokenType.INT);
        else if (JavaConventions.validateIdentifier(val).isOK())
            token = new Token(TokenType.ID);
        else
            token = new Token(TokenType.KEYWORD);
        token.setValue(val);
        return token;
    }
}