/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 19, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: PropertiesView.java,v 1.1 2006/01/23 16:08:08 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.views;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.DrillDownAdapter;
import org.eclipse.ui.part.ViewPart;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.actions.EditPublicFieldAction;
import edu.wpi.ebob.internal.gui.Instance;
import edu.wpi.ebob.internal.inspect.InstanceContentProvider;
import edu.wpi.ebob.internal.inspect.InstanceLabelProvider;
import edu.wpi.ebob.internal.inspect.RootNode;
import edu.wpi.ebob.preferences.OBPreferencePage;

/**
 * This view represents the result of inspecting an instance.
 */
public class PropertiesView extends ViewPart implements ISelectionListener {
    /** the margin height to use when displaying {@link #emptySelectionLabel}. */
    private static final int LABEL_MARGIN_HEIGHT = 2;

    /** the margin width to use when displaying {@link #emptySelectionLabel}. */
    private static final int LABEL_MARGIN_WIDTH = 3;

    private DrillDownAdapter drillDownAdapter;

    private EditPublicFieldAction editFieldsAction;

    /** shown when there is no selection in the Object Bench View. */
    private Label emptySelectionLabel;

    /** the layout for the view. */
    private final StackLayout layout = new StackLayout();

    /** a reference to the parent composite. */
    private Composite panel;

    /** the viewer for the view. */
    private TreeViewer viewer;

    private Instance currentInstance;

    private Preferences.IPropertyChangeListener prefChangeListener;

    public PropertiesView() {
        prefChangeListener = new Preferences.IPropertyChangeListener() {
            public void propertyChange(Preferences.PropertyChangeEvent event) {
                String property = event.getProperty();
                if (OBPreferencePage.P_FQ_CLASS_NAMES.equals(property)) {
                    viewer.refresh(true);
                }
            }
        };
        ObjBenchPlugin.getDefault().getPluginPreferences()
                .addPropertyChangeListener(prefChangeListener);

    }

    /**
     *  
     */
    private void createEmptySelectionLabel() {
        emptySelectionLabel = new Label(panel, SWT.WRAP);
        emptySelectionLabel.setText(ObjBenchPlugin
                .getResourceString("PropertiesView.empty.selection.text")); //$NON-NLS-1$
    }

    /**
     * This is a callback that will allow us to create the viewer and initialize
     * it.
     */
    public void createPartControl(Composite parent) {
        panel = new Composite(parent, SWT.NONE);

        createEmptySelectionLabel();
        createViewer();

        panel.setLayout(layout);

        // fill actions
        hookContextMenu();
        fillLocalToolBar(getViewSite().getActionBars().getToolBarManager());

        // check to see if a selection has already been made
        IWorkbenchPage activePage = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        if (null != activePage) {
            ObjBenchView objBenchView = (ObjBenchView) activePage
                    .findView(ObjBenchView.class.getName());
            if (null != objBenchView)
                selectionChanged(objBenchView, objBenchView.getSite()
                        .getSelectionProvider().getSelection());
        }
        // if top control hasn't already been decided, set it to the empty label
        if (null == layout.topControl) {
            layout.marginWidth = LABEL_MARGIN_WIDTH;
            layout.marginHeight = LABEL_MARGIN_HEIGHT;
            layout.topControl = emptySelectionLabel;
        }
    }

    /**
     * creates the TableTreeViewer for this view.
     */
    private void createViewer() {
        viewer = new TreeViewer(panel, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL);
        viewer.setContentProvider(new InstanceContentProvider());
        viewer.setLabelProvider(new InstanceLabelProvider());

        drillDownAdapter = new DrillDownAdapter(viewer);

        editFieldsAction = new EditPublicFieldAction(viewer);
        viewer.addDoubleClickListener(new IDoubleClickListener() {
            public void doubleClick(DoubleClickEvent event) {
                editFieldsAction.run();
            }
        });
    }

    /**
     * removes the selection listener and the preference listener.
     * 
     * @see IWorkbenchPart#dispose()
     */
    public void dispose() {
        ObjBenchPlugin.getDefault().getPluginPreferences()
                .removePropertyChangeListener(prefChangeListener);
        getViewSite().getPage().removeSelectionListener(
                ObjBenchView.class.getName(), this);
    }

    private void fillLocalToolBar(IToolBarManager manager) {
        drillDownAdapter.addNavigationActions(manager);
    }

    private void hookContextMenu() {
        MenuManager menuMgr = new MenuManager(this.getClass().getName()); //$NON-NLS-1$
        menuMgr.add(editFieldsAction);
        menuMgr.add(new Separator());
        drillDownAdapter.addNavigationActions(menuMgr);
        // Other plug-ins can contribute there actions here
        menuMgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
        viewer.getControl().setMenu(menuMgr.createContextMenu(panel));
        getSite().registerContextMenu(menuMgr, viewer);
    }

    /**
     * @see org.eclipse.ui.IViewPart#init(org.eclipse.ui.IViewSite)
     */
    public void init(IViewSite site) throws PartInitException {
        super.init(site);
        site.getPage().addSelectionListener(ObjBenchView.class.getName(), this);
    }

    /**
     * listens to changes in the {@link ObjBenchView}. If an instance is
     * selected, the tree is displayed; otherwise, a label is displayed.
     * 
     * @param part
     *            must be an instance of {@link ObjBenchView}.
     * @param selection
     *            the currently selected instance in the {@link ObjBenchView}.
     * @see ISelectionListener#selectionChanged(IWorkbenchPart, ISelection)
     */
    public void selectionChanged(IWorkbenchPart part, ISelection selection) {
        assert ObjBenchView.class.getName().equals(part.getSite().getId());

        if (null == viewer)
            // we're not ready yet
            return;

        if (selection.isEmpty()) {
            layout.marginWidth = LABEL_MARGIN_WIDTH;
            layout.marginHeight = LABEL_MARGIN_HEIGHT;
            layout.topControl = emptySelectionLabel;
            panel.layout();
            viewer.setInput(null);
            viewer.getControl().getMenu().setEnabled(false);
        } else {
            layout.marginWidth = 0;
            layout.marginHeight = 0;
            layout.topControl = viewer.getControl();
            currentInstance = (Instance) ((IStructuredSelection) selection)
                    .getFirstElement();
            viewer.setInput(new RootNode(currentInstance));
            viewer.expandToLevel(2);
            panel.layout();
            viewer.getControl().getMenu().setEnabled(true);
        }
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
        viewer.getControl().setFocus();
    }

    /**
     * Refreshes the view. This resets the input, collapses the tree entirely,
     * and then expands the tree 2 levels.
     * 
     * TODO it'd be nice if the actual nodes of the tree would refresh instead
     * of having to reset the input.
     */
    public void refresh() {
        viewer.setInput(new RootNode(currentInstance));
        viewer.expandToLevel(2);
    }
}