/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 20, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *    Justin Braga - Catching CMException
 * 
 * $Id: AddToObjectBenchWizard.java,v 1.1 2006/01/23 16:08:17 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.wizards;

import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.TypeStatus;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.model.ObjectAdditionNotSupportedException;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IClassManager;
import edu.wpi.ebob.model.cm.IConstructor;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.parser.ParserException;
import edu.wpi.ebob.views.ObjBenchView;

/**
 * This wizard guides a user through the process of adding an instance to the
 * Object Bench.
 */
public class AddToObjectBenchWizard extends Wizard implements INewWizard {

    /**
     * The fully qualified classname retrieved from the selection.
     * 
     * @see #init(IWorkbench, IStructuredSelection)
     */
    private String fqClassName;

    private IObject obj;

    private AddToObjectBenchWizardPage page1;

    private AddToObjectBenchWizardPageExtension page2;

    /**
     * the currently selected project, retrieved from the selection.
     * 
     * @see #init(IWorkbench, IStructuredSelection)
     */
    private IJavaProject project;

    /**
     * Constructor for AddToObjectBenchWizard.
     */
    public AddToObjectBenchWizard() {
        setNeedsProgressMonitor(true);
    }

    /**
     * Adding the page to the wizard.
     */

    public void addPages() {
        page1 = new AddToObjectBenchWizardPage(project, fqClassName);
        page2 = new AddToObjectBenchWizardPageExtension();
        addPage(page1);
        addPage(page2);
    }

    /**
     * The worker method. It will find the container, create the file if missing
     * or just replace its contents, and open the editor on the newly created
     * file.
     * 
     * @param args
     * @param constructor
     * @throws RemoteException
     */

    private void doFinish(final String instanceName,
            final IConstructor constructor, final IObject[] args,
            IProgressMonitor monitor) throws CoreException, RemoteException {
        monitor.beginTask(MessageFormat.format(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizard.task"), //$NON-NLS-1$
                new Object[] { instanceName }), 2);
        obj = constructor.newInstance(args);
        monitor.worked(1);

        monitor.setTaskName(MessageFormat.format(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizard.add.instance"), //$NON-NLS-1$
                new Object[] { instanceName }));
        getShell().getDisplay().asyncExec(new Runnable() {
            public void run() {
                try {
                    ObjBenchPlugin.getDefault().getBench().addObject(
                            instanceName, obj);
                } catch (ObjectAdditionNotSupportedException oanse) {
                    MessageDialog
                            .openError(
                                    getShell(),
                                    ObjBenchPlugin
                                            .getResourceString("Dialog.title"), //$NON-NLS-1$
                                    ObjBenchPlugin
                                            .getResourceString("AddToObjectBenchWizard.unique.names")); //$NON-NLS-1$
                    ObjBenchLogger
                            .logError(
                                    ObjBenchPlugin
                                            .getResourceString("AddToObjectBenchWizard.unique.names"), //$NON-NLS-1$
                                    oanse);
                }
            }
        });
        monitor.worked(1);
    }

    /**
     * Retrieves the fully-qualified classname and project from the selection.
     * An object of type IType, ICompilationUnit, or IFile is expected to be
     * found in the selection.
     * 
     * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        Object o = selection.getFirstElement();
        if (o instanceof IType)
            setType((IType) o);
        else if (o instanceof ICompilationUnit)
            setType(((ICompilationUnit) o).findPrimaryType());
        else if (o instanceof IFile) {
            ICompilationUnit compUnit = JavaCore
                    .createCompilationUnitFrom((IFile) o);
            setType(compUnit.findPrimaryType());
        }
    }

    /**
     * This method is called when 'Finish' button is pressed in the wizard. We
     * will create an operation and run it using wizard as execution context.
     */
    public boolean performFinish() {
        project = page1.getProject();
        final String instanceName = page1.getInstanceName();
        IClassManager cm = ObjBenchPlugin.getDefault().getClassManager();
        // test to see if second page has been bypassed with a default
        // constructor; if so, construct it
        boolean secondPageBypassed = page1 == getContainer().getCurrentPage();
        final IConstructor constructor = (secondPageBypassed) ? getConstructor()
                : page2.getConstructor();
        final IObject[] args;

        try {
            args = (secondPageBypassed) ? new IObject[] {} : page2
                    .getArguments();
        } catch (ParserException pe) {
            Throwable cause = pe.getCause();
            if (cause instanceof RemoteException)
                DetailsDialogUtil.openError(getShell(), cause);
            else
                MessageDialog.openError(getShell(), ObjBenchPlugin
                        .getResourceString("Dialog.title.error"), pe //$NON-NLS-1$
                        .getLocalizedMessage());
            return false;
        } catch (ClassCastException cce) {
            DetailsDialogUtil.openError(getShell(), cce);
            return false;
        }
        IRunnableWithProgress op = new IRunnableWithProgress() {
            public void run(IProgressMonitor monitor)
                    throws InvocationTargetException {
                try {
                    doFinish(instanceName, constructor, args, monitor);
                } catch (CoreException ce) {
                    throw new InvocationTargetException(ce);
                } catch (RemoteException cme) {
                    throw new InvocationTargetException(cme);
                } finally {
                    monitor.done();
                }
            }
        };
        try {
            getContainer().run(true, false, op);
        } catch (InterruptedException ie) {
            return false;
        } catch (InvocationTargetException ite) {
            Throwable cause = ite.getTargetException();
            DetailsDialogUtil.openError(getShell(), cause);
            return false;
        }
        try {
            PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                    .getActivePage().showView(ObjBenchView.class.getName());
        } catch (PartInitException pie) {
            ObjBenchLogger
                    .logWarning(
                            ObjBenchPlugin
                                    .getResourceString("AddToObjectBenchWizard.couldNotShowObjectBench"), pie); //$NON-NLS-1$
        }
        return true;
    }

    /**
     * @return the constructor that should be used to construct the instance
     */
    private IConstructor getConstructor() {
        IClassManager cm = ObjBenchPlugin.getDefault().getClassManager();
        ObjectBench bench = ObjBenchPlugin.getDefault().getBench();
        IJavaProject currentProject = cm.getAssociatedProject();
        if (currentProject != null) {
            if (currentProject.getProject() != project.getProject()) {
                cm.disassociateProject();
                bench.removeAll();
                cm.associateProject(project);
            }
        } else
            cm.associateProject(project);
        try {
            IClass type = cm.getClass(page1.getClassName());
            return type.getConstructor(new IClass[] {});
        } catch (ClassNotFoundException cnfe) {
            // shouldn't happen
            ObjBenchLogger.logError(cnfe);
            return null;
        }
    }

    /**
     * Sets this wizard to use <code>project</code> as the default project in
     * the project selection drop-down.
     * 
     * @param project
     *            The desired project
     */
    public void setProject(IJavaProject project) {
        this.project = project;
    }

    /**
     * sets the type for this wizard. This in turn sets the associated project
     * and sets the default class selection.
     * 
     * @param type
     *            the java element to be associated with the project.
     */
    public void setType(IType type) {
        // validate the type
        if (!TypeStatus.getStatus(type).isOK())
            return;
        setProject(type.getJavaProject());
        fqClassName = type.getFullyQualifiedName();
    }
}