/*******************************************************************************
 * Copyright (c) 2004, 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 20, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: AddToObjectBenchWizardPage.java,v 1.1 2006/01/23 16:08:17 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.wizards;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchMatch;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.core.search.SearchRequestor;
import org.eclipse.jdt.ui.IJavaElementSearchConstants;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.TypeStatus;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IClassManager;

/**
 * The "New" wizard page allows setting the container for the new file as well
 * as the file name. The page will only accept file name without the extension
 * OR with the extension that matches the expected one (mpe).
 */

public class AddToObjectBenchWizardPage extends WizardPage {

    private class TypeSearchRequestor extends SearchRequestor {
        private TypeStatus lastStatus;

        private boolean isFullyQualified;

        private String className;

        TypeSearchRequestor(String className) {
            this.className = className;
            updateStatus(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPage.invalid.class")); //$NON-NLS-1$
        }

        public void acceptSearchMatch(SearchMatch match) throws CoreException {
            // verify that this is a public class with a fully qualified name
            IType type = (IType) match.getElement();
            lastStatus = TypeStatus.getStatus(type);
            isFullyQualified = type.getFullyQualifiedName().equals(className);
        }

        public void endReporting() {
            if (isFullyQualified && null != lastStatus) {
                if (lastStatus.isOK())
                    updateStatus(null);
                else
                    updateStatus(lastStatus.getMessage()); //$NON-NLS-1$
            }
        }
    }

    private class ConstructorSearchRequestor extends SearchRequestor {
        private boolean exists = false;

        /**
         * @see org.eclipse.jdt.core.search.SearchRequestor#acceptSearchMatch(org.eclipse.jdt.core.search.SearchMatch)
         */
        public void acceptSearchMatch(SearchMatch match) throws CoreException {
            IMethod constructor = (IMethod) match.getElement();
            // is it a public no-arg constructor?
            if (constructor.getNumberOfParameters() == 0
                    && Flags.isPublic(constructor.getFlags())
                    && constructor.isConstructor())
                exists = true;
        }

        /**
         * @see org.eclipse.jdt.core.search.SearchRequestor#endReporting()
         */
        public void endReporting() {
            ((WizardPage) getNextPage()).setPageComplete(exists);
            getWizard().getContainer().updateButtons();
        }
    }

    private Button browseButton;

    private Text classText;

    private String defaultClassName;

    private Text nameText;

    private IJavaProject project;

    private IJavaSearchScope scope;

    private final SearchEngine se;

    private Button projCheck;

    /**
     * Constructor for SampleNewWizardPage.
     * 
     * @param project
     */
    public AddToObjectBenchWizardPage(IJavaProject project, String className) {
        super(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPage.name")); //$NON-NLS-1$
        setTitle(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPage.title")); //$NON-NLS-1$
        setDescription(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPage.description")); //$NON-NLS-1$
        defaultClassName = (className != null) ? className : ""; //$NON-NLS-1$
        this.project = project;
        se = new SearchEngine();
        scope = SearchEngine
                .createJavaSearchScope(new IJavaElement[] { project });
    }

    /**
     * Creates the "Class name" text row
     * 
     * @param parent
     *            The composite parent that this row will be added to. It's
     *            assumed that it uses GridLayout.
     */
    private void createClassText(Composite parent) {
        Label label = new Label(parent, SWT.LEFT | SWT.WRAP);
        label.setText(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPage.class.name")); //$NON-NLS-1$
        label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));

        classText = new Text(parent, SWT.BORDER | SWT.SINGLE);
        classText.setText(defaultClassName);
        GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
        gd.grabExcessHorizontalSpace = true;
        classText.setLayoutData(gd);

        classText.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                dialogChanged();
            }
        });

        browseButton = new Button(parent, SWT.PUSH);
        browseButton.setFont(JFaceResources.getDialogFont());
        browseButton.setText(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPage.browse")); //$NON-NLS-1$
        browseButton.setEnabled(project != null);
        browseButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                handleBrowse();
            }
        });

        initializeDialogUnits(browseButton);
        int desiredWidth = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);

        gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
        gd.widthHint = Math.max(desiredWidth, browseButton.computeSize(
                SWT.DEFAULT, SWT.DEFAULT, true).x);
        gd.heightHint = convertVerticalDLUsToPixels(IDialogConstants.BUTTON_HEIGHT);
        browseButton.setLayoutData(gd);

        initializeDialogUnits(parent);
        ((GridData) classText.getLayoutData()).widthHint = convertWidthInCharsToPixels(40);
    }

    /**
     * @see IDialogPage#createControl(Composite)
     */
    public void createControl(Composite parent) {
        try {
            Composite container = new Composite(parent, SWT.NONE);
            GridLayout layout = new GridLayout();
            layout.numColumns = 3;
            container.setLayout(layout);
            //layout.verticalSpacing = 9;

            createProjectCombo(container);
            createNameText(container);
            createClassText(container);

            setPageComplete(false);
            setControl(container);
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    /**
     * Creates the "Instance name" text row
     * 
     * @param parent
     *            The composite parent that this row will be added to. It's
     *            assumed that it uses GridLayout.
     */
    private void createNameText(Composite parent) {
        Label label = new Label(parent, SWT.LEFT | SWT.WRAP);
        label.setText(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPage.instance.name")); //$NON-NLS-1$
        label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));

        nameText = new Text(parent, SWT.BORDER | SWT.SINGLE);
        nameText.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
                | GridData.FILL_HORIZONTAL));

        // if project has already been set, set focus here
        if (null != project)
            nameText.setFocus();

        nameText.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                dialogChanged();
            }
        });

        // this button only exists for filling up a cell
        Label hiddenLabel = new Label(parent, SWT.NONE);
        hiddenLabel.setVisible(false);
        hiddenLabel.setCapture(false);

        initializeDialogUnits(parent);
        ((GridData) nameText.getLayoutData()).widthHint = convertWidthInCharsToPixels(40);
    }

    /**
     * @param parent
     */
    private void createProjectCombo(Composite parent) {
        IProject[] projects = ResourcesPlugin.getWorkspace().getRoot()
                .getProjects();
        List javaProjectList = new ArrayList();
        for (int i = 0; i < projects.length; i++) {
            IProject project = projects[i];
            try {
                if (project.hasNature(JavaCore.NATURE_ID))
                    javaProjectList.add(project);
            } catch (CoreException ce) {
                ObjBenchLogger
                        .logWarning(
                                MessageFormat
                                        .format(
                                                ObjBenchPlugin
                                                        .getResourceString("AddToObjectBenchWizardPage.nature.missing"), //$NON-NLS-1$
                                                new Object[] { project
                                                        .getName() }), ce);
            }
        }
        IProject[] javaProjects = new IProject[javaProjectList.size()];
        for (int i = 0; i < javaProjectList.size(); i++)
            javaProjects[i] = (IProject) javaProjectList.get(i);

        Label label = new Label(parent, SWT.LEFT | SWT.WRAP);
        label.setText(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPage.proj.name")); //$NON-NLS-1$
        label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        Combo projCombo = new Combo(parent, SWT.READ_ONLY);

        GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL
                | GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        projCombo.setLayoutData(gd);
        ComboViewer projComboViewer = new ComboViewer(projCombo);
        projComboViewer.setContentProvider(new ArrayContentProvider());
        projComboViewer.setLabelProvider(new WorkbenchLabelProvider());
        projComboViewer.setInput(javaProjects);

        if (null != project)
            projComboViewer.setSelection(new StructuredSelection(project
                    .getProject()));

        projComboViewer
                .addSelectionChangedListener(new ISelectionChangedListener() {
                    public void selectionChanged(SelectionChangedEvent event) {
                        IProject proj = (IProject) ((IStructuredSelection) event
                                .getSelection()).getFirstElement();
                        setProject(JavaCore.create(proj));

                        IClassManager cm = ObjBenchPlugin.getDefault()
                                .getClassManager();
                        IJavaProject currentProject = cm.getAssociatedProject();

                        projCheck.setEnabled(null != currentProject
                                && currentProject.getProject() != getProject()
                                        .getProject());
                        
                        dialogChanged();
                    }
                });

        // fill up a cell
        Label hiddenLabel = new Label(parent, SWT.NONE);
        hiddenLabel.setVisible(false);
        hiddenLabel.setCapture(false);

        projCheck = new Button(parent, SWT.CHECK);
        projCheck
                .setText(ObjBenchPlugin
                        .getResourceString("AddToObjectBenchWizardPage.projCheckLabel")); //$NON-NLS-1$
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        projCheck.setLayoutData(gd);

        // set enabled only if the current project isn't the selected project
        if (null != project) {
            IClassManager cm = ObjBenchPlugin.getDefault().getClassManager();
            IJavaProject currentProject = cm.getAssociatedProject();

            projCheck.setEnabled(null != currentProject
                    && currentProject.getProject() != project.getProject());
        } else
            projCheck.setEnabled(false);

        projCheck.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent se) {
                if (projCheck.getSelection()) {
                    if (MessageDialog
                            .openConfirm(
                                    getShell(),
                                    ObjBenchPlugin
                                            .getResourceString("AddToObjectBenchWizardPage.reassociationTitle"), //$NON-NLS-1$
                                    ObjBenchPlugin
                                            .getResourceString("AddToObjectBenchWizardPage.reassociationMessage"))) //$NON-NLS-1$
                        dialogChanged();
                    else
                        projCheck.setSelection(false);
                }
            }
        });
    }

    /**
     * Ensures that both text fields are set.
     */

    private void dialogChanged() {
        String instanceName = nameText.getText();
        String className = classText.getText();

        if (null == project) {
            updateStatus(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPage.project.must.be.selected")); //$NON-NLS-1$
            return;
        }

        if (instanceName.length() == 0) {
            updateStatus(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPage.instance.must.be.specified")); //$NON-NLS-1$
            return;
        }

        ObjectBench bench = ObjBenchPlugin.getDefault().getBench();
        if (null != bench && bench.containsObject(instanceName)) {
            updateStatus(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPage.no.duplicates")); //$NON-NLS-1$
            return;
        }

        IStatus validIdentifier = JavaConventions
                .validateIdentifier(instanceName);
        if (!validIdentifier.isOK()) {
            updateStatus(validIdentifier.getMessage());
            return;
        }

        if (className.length() == 0) {
            updateStatus(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPage.type.must.be.specified")); //$NON-NLS-1$
            return;
        }

        if (projCheck.isEnabled() && !projCheck.getSelection()) {
            updateStatus(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPage.reassociationWarning")); //$NON-NLS-1$
            return;
        }

        SearchPattern pattern = SearchPattern.createPattern(className,
                IJavaSearchConstants.TYPE, IJavaSearchConstants.DECLARATIONS,
                SearchPattern.R_EXACT_MATCH | SearchPattern.R_CASE_SENSITIVE);

        if (pattern != null) {
            try {
                TypeSearchRequestor searchRequestor = new TypeSearchRequestor(
                        className);
                se.search(pattern, new SearchParticipant[] { SearchEngine
                        .getDefaultSearchParticipant() }, scope,
                        searchRequestor, null);
            } catch (CoreException ce) {
                ObjBenchLogger.logError(ce);
            }
        } else {
            updateStatus(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPage.invalid.class")); //$NON-NLS-1$
            return;
        }

        if (isPageComplete()) {
            ConstructorSearchRequestor constSearcher = new ConstructorSearchRequestor();
            SearchPattern constPattern = SearchPattern.createPattern(className,
                    IJavaSearchConstants.CONSTRUCTOR,
                    IJavaSearchConstants.DECLARATIONS,
                    SearchPattern.R_PREFIX_MATCH
                            | SearchPattern.R_CASE_SENSITIVE);
            try {
                se.search(constPattern, new SearchParticipant[] { SearchEngine
                        .getDefaultSearchParticipant() }, scope, constSearcher,
                        null);
            } catch (CoreException ce) {
                ce.printStackTrace();
                ObjBenchLogger.logError(ce);
            }
        }
    }

    public String getInstanceName() {
        return nameText.getText();
    }

    public String getClassName() {
        return classText.getText();
    }

    public IJavaProject getProject() {
        return project;
    }

    /**
     * Uses the standard container selection dialog to choose the new value for
     * the container field.
     */
    private void handleBrowse() {
        try {
            SelectionDialog dialog = JavaUI.createTypeDialog(getShell(),
                    getWizard().getContainer(), scope,
                    IJavaElementSearchConstants.CONSIDER_CLASSES,
                    false /* multipleSelection */, classText.getText());
            dialog
                    .setTitle(ObjBenchPlugin
                            .getResourceString("AddToObjectBenchWizardPage.class.selection")); //$NON-NLS-1$
            dialog
                    .setMessage(ObjBenchPlugin
                            .getResourceString("AddToObjectBenchWizardPage.choose.type")); //$NON-NLS-1$
            if (dialog.open() == Window.OK) {
                Object[] result = dialog.getResult();

                if (result.length == 1)
                    classText.setText(((IType) result[0])
                            .getFullyQualifiedName());
            }
        } catch (JavaModelException jme) {
            ObjBenchLogger.logError(jme);
        }
    }

    public void setProject(IJavaProject project) {
        this.project = project;
        scope = SearchEngine
                .createJavaSearchScope(new IJavaElement[] { project });
        browseButton.setEnabled(project != null);
        dialogChanged();
    }

    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#setVisible(boolean)
     */
    public void setVisible(boolean visible) {
        if (!visible) {
            IClassManager cm = ObjBenchPlugin.getDefault().getClassManager();
            IJavaProject currentProject = cm.getAssociatedProject();
            if (currentProject != null) {
                if (currentProject.getProject() != project.getProject()) {
                    cm.disassociateProject();
                    ObjBenchPlugin.getDefault().getBench().removeAll();
                    cm.associateProject(project);
                }
            } else
                cm.associateProject(project);
            AddToObjectBenchWizardPageExtension page2 = (AddToObjectBenchWizardPageExtension) getNextPage();
            IClass type = null;
            try {
                type = cm.getClass(classText.getText());
            } catch (ClassNotFoundException cnfe) {
                // this should never happen, log as an error
                ObjBenchLogger.logError(cnfe);
            }
            assert null != type;
            page2.setInstanceClass(type);
        }
        super.setVisible(visible);
    }

    private void updateStatus(String message) {
        setErrorMessage(message);
        if (null == message
                && ObjBenchPlugin.getDefault().getClassManager().isDirty())
            setMessage(
                    ObjBenchPlugin
                            .getResourceString("AddToObjectBenchWizardPage.dirtyWarning"), WARNING); //$NON-NLS-1$
        setPageComplete(message == null);
    }
}